/**********************************************************************
Copyright (C) 2005 Mustek Corporation, All rights reserved
    
Filename: MustScanner.c   [Abstract: ]

Author: Jack            Date: 2005/05/13
	
Function Description:
	Scan the image(reflective or transparent)

[Operative Environment:]

[Reference Articles:]

Revision History:
*********************************************************************/
/* sane - Scanner Access Now Easy.

   Copyright (C) 2005 Mustek.
   Originally maintained by Mustek
   Author:Roy 2005.5.24

   This file is part of the SANE package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   As a special exception, the authors of SANE give permission for
   additional uses of the libraries contained in this release of SANE.

   The exception is that, if you link a SANE library with other files
   to produce an executable, this does not by itself cause the
   resulting executable to be covered by the GNU General Public
   License.  Your use of that executable is in no way restricted on
   account of linking the SANE library code into it.

   This exception does not, however, invalidate any other reasons why
   the executable file might be covered by the GNU General Public
   License.

   If you submit changes to SANE to the maintainers to be included in
   a subsequent release, you agree by submitting the changes that
   those changes may be distributed with this exception intact.

   If you write modifications of your own for SANE, it is your choice
   whether to permit this exception to apply to your modifications.
   If you do not wish that, delete this exception notice.

*/
#include <pthread.h>
#include <stdlib.h>

//local header file
#include "../include/typedef.h"
#include "../include/asic.h"
#include "../include/public.h"
#include "../include/spicall.h"
#include "../include/ini.h"
#include "../include/debug.h"
#include "MustScanner.h"
#include "ScannerDefine.h"
#include "spicall_g.h"

extern unsigned short*	g_pGammaTable;
extern char*			g_pDeviceFile;
extern WORD				g_wLineartThreshold;
extern BOOL				g_bSharpen;

int						g_nSecLength, g_nDarkSecLength;
int						g_nSecNum, g_nDarkSecNum;
WORD					g_wCalWidth;
WORD					g_wDarkCalWidth;
int						g_nPowerNum;
WORD					g_wStartPosition;

pthread_mutex_t			g_scannedLinesMutex = PTHREAD_MUTEX_INITIALIZER; 
pthread_mutex_t			g_readyLinesMutex   = PTHREAD_MUTEX_INITIALIZER; 

DWORD GetScannedLines();
DWORD GetReadyLines();
void  AddScannedLines(WORD wAddLines);
void  AddReadyLines();

///////////////////////////////////////////////
//function description
///////////////////////////////////////////////

/**********************************************************************
Author: Jack            Date: 2005/05/13
	Routine Description: 
Parameters:
	none
Return value: 
	if initialize the scanner success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL MustScanner_Init()
{
	XDBG(DEBUG_SPICALL,"MustScanner_Init: Call in");

	//Initialize the firmware status
	g_chip.firmwarestate = FS_NULL;
	if(STATUS_GOOD != Asic_Open(&g_chip, g_pDeviceFile))
	{
		XDBG(DEBUG_SPICALL,"MustScanner_Init: Asic_Open return error");
		return FALSE;
	}

	if(STATUS_GOOD != Asic_Initialize(&g_chip))
	{	
		XDBG(DEBUG_SPICALL,"MustScanner_Init: Asic_Initialize return error");
		return FALSE;
	}

	g_dwImageBufferSize = 24L*1024L*1024L;
	g_dwBufferSize		= 64L*1024L;
	g_dwCalibrationSize = 64L*1024L;
	g_lpReadImageHead	= NULL;

	g_isCanceled		= FALSE;
	g_bFirstReadImage	= TRUE;
	g_bOpened			= FALSE;
	g_bPrepared			= FALSE;
	g_bSharpen			= FALSE;

	g_isScanning		= FALSE;
	g_isSelfGamma		= FALSE;
	g_pGammaTable		= NULL;

	if(NULL != g_pDeviceFile)
	{
		free(g_pDeviceFile);
		g_pDeviceFile = NULL;
	}

	g_ssScanSource = SS_Reflective;
	g_PixelFlavor  = PF_BlackIs0;

	Asic_Close(&g_chip);	
	
	XDBG(DEBUG_SPICALL,"MustScanner_Init: leave MustScanner_Init");

	return TRUE;
}

/**********************************************************************
Author: Jack            Date: 2005/05/13
Routine Description: 
	check the scanner connect status
Parameters:
	none
Return value: 
	if scanner's status is OK
	return TRUE
	else 
	return FASLE
***********************************************************************/
BOOL MustScanner_GetScannerState()
{
	/* //not need check,Asic_Open can receive NULL parameter
	if(NULL == g_pDeviceFile)
	{
		return FALSE;
	}
	*/

	if(STATUS_GOOD != Asic_Open(&g_chip, g_pDeviceFile))
	{
		XDBG(DEBUG_SPICALL,"MustScanner_GetScannerState: Asic_Open return error");
		return FALSE;
	}
	else
	{
		Asic_Close(&g_chip);
		return TRUE;
	}	
}

/**********************************************************************
Author: Jack            Date: 2005/05/13
Routine Description: 
	Turn the lamp on or off
Parameters:
	isLampOn: turn the lamp on or off
	isTALampOn: turn the TA lamp on or off
Return value: 
	if operation success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL MustScanner_PowerControl(BOOL isLampOn, BOOL isTALampOn)
{
	XDBG(DEBUG_SPICALL,"MustScanner_PowerControl: Call in");
	BOOL hasTA;
	if(STATUS_GOOD != Asic_Open(&g_chip, g_pDeviceFile))
	{
		XDBG(DEBUG_SPICALL,"MustScanner_PowerControl: Asic_Open return error");
		return FALSE;
	}

	if(STATUS_GOOD != Asic_TurnLamp(&g_chip, isLampOn))
	{	
		XDBG(DEBUG_SPICALL,"MustScanner_PowerControl: Asic_TurnLamp return error");
		return FALSE;
	}

	if(STATUS_GOOD != Asic_IsTAConnected(&g_chip, &hasTA))
	{
		XDBG(DEBUG_SPICALL,"MustScanner_PowerControl: Asic_IsTAConnected return error");
		return FALSE;
	}

	if (hasTA)
	{
		if (STATUS_GOOD != Asic_TurnTA(&g_chip, isTALampOn))
		{
			XDBG(DEBUG_SPICALL,"MustScanner_PowerControl: Asic_TurnTA return error");
			return FALSE;
		}
	}

	Asic_Close(&g_chip);

	XDBG(DEBUG_SPICALL,"MustScanner_PowerControl: leave MustScanner_PowerControl");
	return TRUE;
}

/**********************************************************************
Author: Jack            Date: 2005/05/13
Routine Description: 
	Turn the carriage home
Parameters:
	none
Return value: 
	if the operation success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL MustScanner_BackHome()
{
	XDBG(DEBUG_SPICALL,"MustScanner_BackHome: call in ");

	if(STATUS_GOOD != Asic_Open(&g_chip, g_pDeviceFile))
	{
    	XDBG(DEBUG_SPICALL,"MustScanner_BackHome: Asic_Open return error");
		return FALSE;
	}

	if(STATUS_GOOD != Asic_CarriageHome(&g_chip, FALSE))
	{
    	XDBG(DEBUG_SPICALL,"MustScanner_BackHome: Asic_CarriageHome return error");
		return FALSE;
	}

	if(STATUS_GOOD != Asic_WaitUnitReady(&g_chip))
	{
    	XDBG(DEBUG_SPICALL,"MustScanner_BackHome: Asic_WaitUnitReady return error");
		return FALSE;
	}

	Asic_Close(&g_chip);
	
	XDBG(DEBUG_SPICALL,"MustScanner_BackHome: leave  MustScanner_BackHome");
	return TRUE;
}

/**********************************************************************
Author: Jack            Date: 2005/05/13
Routine Description: 
	prepare the scan image
Parameters:
	bScanSource: the scan source
Return value: 
	if operation is success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL MustScanner_Prepare(BYTE bScanSource)
{	
	XDBG(DEBUG_SPICALL,"MustScanner_Prepare: call in");

	if(STATUS_GOOD != Asic_Open(&g_chip, g_pDeviceFile))
	{
		XDBG(DEBUG_SPICALL,"MustScanner_Prepare: Asic_Open return error");
		return FALSE;
	}
	if(STATUS_GOOD != Asic_WaitUnitReady(&g_chip))
	{
		XDBG(DEBUG_SPICALL,"MustScanner_Prepare: Asic_WaitUnitReady return error");
		return FALSE;
	}

	if(SS_Reflective == bScanSource)
	{
		XDBG(DEBUG_SPICALL,"MustScanner_Prepare:ScanSource is SS_Reflective");
		if(STATUS_GOOD != Asic_TurnLamp(&g_chip, TRUE))
		{	
			XDBG(DEBUG_SPICALL,"MustScanner_Prepare: Asic_TurnLamp return error");
			return FALSE;
		}

		if(STATUS_GOOD != Asic_SetSource(&g_chip,LS_REFLECTIVE))
		{
			XDBG(DEBUG_SPICALL,"MustScanner_Prepare: Asic_SetSource return error");
			return FALSE;
		}
	}
	else if(SS_Positive == bScanSource)
	{
		XDBG(DEBUG_SPICALL,"MustScanner_Prepare:ScanSource is SS_Positive");
		if(STATUS_GOOD != Asic_TurnTA(&g_chip,TRUE))
		{
			XDBG(DEBUG_SPICALL,"MustScanner_Prepare: Asic_TurnTA return error");
			return FALSE;
		}
		if(STATUS_GOOD != Asic_SetSource(&g_chip,LS_POSITIVE))
		{
			XDBG(DEBUG_SPICALL,"MustScanner_Prepare: Asic_SetSource return error");
			return FALSE;
		}
	}
	else if(SS_Negative == bScanSource)
	{	
		XDBG(DEBUG_SPICALL,"MustScanner_Prepare:ScanSource is SS_Negative");

		if(STATUS_GOOD != Asic_TurnTA(&g_chip,TRUE))
		{
			XDBG(DEBUG_SPICALL,"MustScanner_Prepare: Asic_TurnTA return error");
			return FALSE;
		}

		if(STATUS_GOOD != Asic_SetSource(&g_chip,LS_NEGATIVE))
		{
			XDBG(DEBUG_SPICALL,"MustScanner_Prepare: Asic_SetSource return error");
			return FALSE;
		}
		XDBG(DEBUG_SPICALL,"MustScanner_Prepare: Asic_SetSource return good");
	}

	Asic_Close(&g_chip);
	g_bPrepared = TRUE;

	XDBG(DEBUG_SPICALL,"MustScanner_Prepare: leave MustScanner_Prepare");
	return TRUE;
}

/**********************************************************************
Author: Jack             Date: 2005/05/15
Routine Description: 
	Adjuest the offset
Parameters:
	nTimes: Adjuest offset the times
	bDirection: whether direction
	bOffset: the data of offset
	bLastMin: the last min data
	bLastOffset: the last offset data
	wMinValue: the min value of offset
	bOffsetUpperBound: the upper bound of offset
	bOffsetLowerBound: the lower bound of offset
	wStdMinLevel: the min level of offset
	wStdMaxLevel: the max level of offset
Return value: 
	if the operation is success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL MustScanner_AdjustOffset(int nTimes,BOOL *bDirection,BYTE *bOffset,
							  BYTE *bLastMin,BYTE *bLastOffset,WORD *wMinValue, 
							  BYTE *bOffsetUpperBound, BYTE *bOffsetLowerBound, 
							  WORD wStdMinLevel,WORD wStdMaxLevel)
{
	if((*wMinValue <= wStdMaxLevel) && (*wMinValue >= wStdMinLevel))
	{
		return TRUE;
	}
	
	if(nTimes == 0)
	{
		*bLastMin = LOBYTE(*wMinValue);
		*bLastOffset = *bOffset;
		
		if(*wMinValue == 255)  
		{
			*bOffset = 0; 
		}
		else
		{
			*bOffset = 255; 
		}
	}

	if(nTimes == 1)
	{
		// choose the driection
		if(*wMinValue > *bLastMin)
		{
			if(*wMinValue > wStdMaxLevel && *bLastMin > wStdMaxLevel)
			{
				*bDirection = !(*bDirection); 
				return TRUE;
			}
			
			if(*wMinValue < wStdMinLevel && *bLastMin < wStdMinLevel)
				return TRUE;
			//*bDirection = !(*bDirection); 
		}
							
		if(*wMinValue < *bLastMin)
		{
			if(*wMinValue < wStdMinLevel && *bLastMin < wStdMinLevel)
				*bDirection = !(*bDirection); 
			
			if(*wMinValue > wStdMaxLevel && *bLastMin > wStdMaxLevel)
				return TRUE;
		}
	}
	
	if(nTimes > 1)
	{
		if(*wMinValue > *bLastMin)
		{
			BYTE bTemp;
			
			bTemp = *bOffset;
			
			*bOffset =  (*bOffsetUpperBound + *bOffsetLowerBound) / 2;
			
			if(nTimes > 2)
			{
				if(*wMinValue > wStdMaxLevel)
					if(bDirection)
						*bOffsetLowerBound = bTemp;
					else
						*bOffsetUpperBound = bTemp;
					else
						if(bDirection)
							*bOffsetUpperBound = bTemp;
						else
							*bOffsetLowerBound = bTemp;
			}
			
			*bLastOffset = bTemp;
			*bLastMin = (BYTE) *wMinValue;
		}
		else
		{
			BYTE bTemp;
			
			bTemp = *bOffset;
			
			*bOffset =  (*bOffsetUpperBound + *bOffsetLowerBound) / 2;
			
			if(nTimes > 2)
			{
				if(*wMinValue == *bLastMin)
				{
					if(*wMinValue > wStdMaxLevel)
					{
						if(!bDirection)
							*bOffsetUpperBound = bTemp;
						else
							*bOffsetLowerBound = bTemp;
					}
					else
					{
						if(!bDirection)
							*bOffsetLowerBound = bTemp;
						else
							*bOffsetUpperBound = bTemp;
					}
				}
				else
				{
					if(*wMinValue > wStdMaxLevel)
					{
						if(bDirection)
							*bOffsetUpperBound = bTemp;
						else
							*bOffsetLowerBound = bTemp;
					}
					else
					{
						if(bDirection)
							*bOffsetLowerBound = bTemp;
						else
							*bOffsetUpperBound = bTemp;
					}
				}
			}
			
			*bLastOffset = bTemp;
			*bLastMin = (BYTE) *wMinValue;
			
		}
	} // end of if(nTimes > 1)

	return TRUE;
}

/**********************************************************************
Author: Jack             Date: 2005/05/15
Routine Description: 
	Adjuest the offset second times
Parameters:
	nTimes: Adjuest offset the times
	bDirection: whether direction
	bOffset: the data of offset
	bLastMin: the last min data
	bLastOffset: the last offset data
	wMinValue: the min value of offset
	bOffsetUpperBound: the upper bound of offset
	bOffsetLowerBound: the lower bound of offset
	wStdMinLevel: the min level of offset
	wStdMaxLevel: the max level of offset
Return value: 
	if the operation is success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL MustScanner_SecondAdjustOffset(int nTimes, BOOL *bDirection, BYTE *bOffset, 
									BYTE *bLastMin, BYTE *bLastOffset, WORD *wMinValue, 
									BYTE *bOffsetUpperBound, BYTE *bOffsetLowerBound, 
									WORD wStdMinLevel,WORD wStdMaxLevel)
{
	if((*wMinValue <= wStdMaxLevel) && (*wMinValue >= wStdMinLevel))
	{
		return TRUE;
	}
	if(nTimes == 0)
	{
		*bLastMin = LOBYTE(*wMinValue);
		*bLastOffset = *bOffset;
		
		if(*bDirection == 0)
		{
			*bOffsetUpperBound = *bLastOffset;
			*bOffsetLowerBound = 0;
			*bOffset = 0; 
		}
		else
		{
			*bOffsetUpperBound = 255;
			*bOffsetLowerBound = *bLastOffset;
			*bOffset = 255; 
		}
	}
	
	if(nTimes >= 1)
	{
		if(*wMinValue > wStdMaxLevel)
		{
			if(*wMinValue > *bLastMin)
			{
				if(*bDirection == 0)
				{
					*bOffsetUpperBound = *bOffset;
//					*bOffsetLowerBound = *bOffset;
//					*bOffset = 0; 
				}
				else
				{
//					*bOffsetUpperBound = *bOffset;
					*bOffsetLowerBound = *bOffset;
//					*bOffset = 255; 
				}
				
				*bOffset = (*bOffsetUpperBound + *bOffsetLowerBound) / 2; 
//				*bDirection = !(*bDirection);
			}
			else
			{
				if(*bDirection == 1)
				{
					*bOffsetUpperBound = *bOffset;
					*bOffset = (*bOffsetUpperBound + *bOffsetLowerBound) / 2; 
				}
				else
				{
					*bOffsetLowerBound = *bOffset;
					*bOffset = (*bOffsetUpperBound + *bOffsetLowerBound) / 2; 
				}
			}
		} //end of if(*wMinValue > MAX_OFFSET)
		
		
		if(*wMinValue < wStdMinLevel)
		{
			if(*wMinValue > *bLastMin)
			{
				if(*bDirection == 0)
				{
//					*bOffsetUpperBound = 255;
					*bOffsetLowerBound = *bOffset;
//					*bOffset = 255; 
				}
				else
				{
					*bOffsetUpperBound = *bOffset;
//					*bOffsetLowerBound = 255;
//					*bOffset = 0; 
				}
				
				*bOffset = (*bOffsetUpperBound + *bOffsetLowerBound) / 2; 
//				*bDirection = !(*bDirection);
			}
			else
			{
				if(*bDirection == 1)
				{
					*bOffsetUpperBound = *bOffset;
					*bOffset = (*bOffsetUpperBound + *bOffsetLowerBound) / 2; 
				}
				else
				{
					*bOffsetLowerBound = *bOffset;
					*bOffset = (*bOffsetUpperBound + *bOffsetLowerBound) / 2; 
				}
			}
		}  //end of if(*wMinValue > MIN_OFFSET)
		*bLastMin = (BYTE) *wMinValue;
	}  //end of if(nTimes >= 1)	
}

/**********************************************************************
Author: Jack             Date: 2005/05/15
Routine Description: 
	Filter the data
Parameters:
	pSort: the sort data
	TotalCount: the total count 
	LowCount: the low count
	HighCount: the upper count
Return value: 
	the data of Filter 	
***********************************************************************/
WORD MustScanner_FiltLower(WORD* pSort,WORD TotalCount,WORD LowCount,WORD HighCount)
{
	WORD Bound=TotalCount-1;
	WORD LeftCount=HighCount-LowCount;
	int Temp=0;
	DWORD Sum=0;
	WORD i,j;

	for (i=0;i<Bound;i++)
	{
		for (j=0;j<Bound-i;j++) 
		{
			if (pSort[j+1]>pSort[j]) 
			{
				Temp=pSort[j];
				pSort[j]=pSort[j+1];
				pSort[j+1]=Temp;
			}
		}
	}

	for (i=0 ; i<LeftCount ; i++)
		Sum+=pSort[i+LowCount];
	return (WORD)(Sum/LeftCount);
}

/**********************************************************************
Author: Jack             Date: 2005/05/15
Routine Description: 
	Repair line when single CCD and color is 48bit
Parameters:
	lpLine: point to image be repaired
	isOrderInvert: RGB or BGR
	wLinesCount: how many line be repaired
Return value: 
	if the operation is success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL MustScanner_GetRgb48BitLine(BYTE* lpLine,BOOL isOrderInvert,WORD *wLinesCount)
{
	XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine: call in ");

	WORD wWantedTotalLines;
	WORD TotalXferLines;
	WORD wRLinePos=0;
	WORD wGLinePos=0;
	WORD wBLinePos=0;
	WORD wRTempData;
	WORD wGTempData;
	WORD wBTempData;
	WORD i;

	g_isCanceled=FALSE;
	g_isScanning=TRUE;
	wWantedTotalLines=*wLinesCount;
	TotalXferLines=0;	

	if(g_bFirstReadImage)
	{
		pthread_create(&g_threadid_readimage,NULL,MustScanner_ReadDataFromScanner,NULL);
		XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine: thread create");
		g_bFirstReadImage=FALSE;
	}

	if(!isOrderInvert)
	{
		for ( ; TotalXferLines<wWantedTotalLines; ) 
		{
			if(g_dwTotalTotalXferLines>=g_SWHeight)
			{
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine: thread exit");

				*wLinesCount = TotalXferLines;
				g_isScanning=FALSE;
				return TRUE;
			}

			if(GetScannedLines() > g_wtheReadyLines)
			{
				wRLinePos=g_wtheReadyLines%g_wMaxScanLines;
				wGLinePos=(g_wtheReadyLines-g_wLineDistance)%g_wMaxScanLines;
				wBLinePos=(g_wtheReadyLines-g_wLineDistance*2)%g_wMaxScanLines;
				
				for(i=0; i<g_SWWidth; i++)
				{
					wRTempData =*(g_lpReadImageHead+wRLinePos*g_BytesPerRow+i*6+0);		//R-channel
					wRTempData+=*(g_lpReadImageHead+wRLinePos*g_BytesPerRow+i*6+1)<<8;	//R-channel
					wGTempData =*(g_lpReadImageHead+wGLinePos*g_BytesPerRow+i*6+2);		//G-channel
					wGTempData+=*(g_lpReadImageHead+wGLinePos*g_BytesPerRow+i*6+3)<<8;	//G-channel
					wBTempData =*(g_lpReadImageHead+wBLinePos*g_BytesPerRow+i*6+4);		//B-channel
					wBTempData+=*(g_lpReadImageHead+wBLinePos*g_BytesPerRow+i*6+5)<<8;	//B-channel
					*(lpLine+i*6+0)=LOBYTE(g_pGammaTable[wRTempData]);
					*(lpLine+i*6+1)=HIBYTE(g_pGammaTable[wRTempData]);
					*(lpLine+i*6+2)=LOBYTE(g_pGammaTable[wGTempData+65536]);
					*(lpLine+i*6+3)=HIBYTE(g_pGammaTable[wGTempData+65536]);
					*(lpLine+i*6+4)=LOBYTE(g_pGammaTable[wBTempData+131072]);
					*(lpLine+i*6+5)=HIBYTE(g_pGammaTable[wBTempData+131072]);
				}					
				TotalXferLines++;
				g_dwTotalTotalXferLines++;
//				lpLine+=g_BytesPerRow;
				lpLine+=g_SWBytesPerRow;
//				g_wtheReadyLines++;
				AddReadyLines();
			}

			if(g_isCanceled)
			{
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);

				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine: thread exit");

				break;
			}
		}//end for
	}
	else		
	{
		for ( ; TotalXferLines<wWantedTotalLines; ) 
		{
			if(g_dwTotalTotalXferLines>=g_SWHeight)
			{
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);

				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine: thread exit");

				*wLinesCount = TotalXferLines;
				g_isScanning=FALSE;
				return TRUE;
			}

			if(GetScannedLines() > g_wtheReadyLines)
			{
				wRLinePos=g_wtheReadyLines%g_wMaxScanLines;
				wGLinePos=(g_wtheReadyLines-g_wLineDistance)%g_wMaxScanLines;
				wBLinePos=(g_wtheReadyLines-g_wLineDistance*2)%g_wMaxScanLines;
				
				for(i=0; i<g_SWWidth; i++)
				{
					wRTempData =*(g_lpReadImageHead+wRLinePos*g_BytesPerRow+i*6+0); //R-channel
					wRTempData+=*(g_lpReadImageHead+wRLinePos*g_BytesPerRow+i*6+1)<<8; //R-channel
					wGTempData =*(g_lpReadImageHead+wGLinePos*g_BytesPerRow+i*6+2); //G-channel
					wGTempData+=*(g_lpReadImageHead+wGLinePos*g_BytesPerRow+i*6+3)<<8; //G-channel
					wBTempData =*(g_lpReadImageHead+wBLinePos*g_BytesPerRow+i*6+4); //B-channel
					wBTempData+=*(g_lpReadImageHead+wBLinePos*g_BytesPerRow+i*6+5)<<8; //B-channel
					*(lpLine+i*6+4)=LOBYTE(g_pGammaTable[wRTempData]);
					*(lpLine+i*6+5)=HIBYTE(g_pGammaTable[wRTempData]);
					*(lpLine+i*6+2)=LOBYTE(g_pGammaTable[wGTempData+65536]);
					*(lpLine+i*6+3)=HIBYTE(g_pGammaTable[wGTempData+65536]);
					*(lpLine+i*6+0)=LOBYTE(g_pGammaTable[wBTempData+131072]);
					*(lpLine+i*6+1)=HIBYTE(g_pGammaTable[wBTempData+131072]);
				}					
				
				TotalXferLines++;
				g_dwTotalTotalXferLines++;
//				lpLine+=g_BytesPerRow;
				lpLine+=g_SWBytesPerRow;
//				g_wtheReadyLines++;
				AddReadyLines();
				
			}
			if(g_isCanceled)
			{	
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);

				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine: thread exit");
				break;
			}
		}//end for
	}	

	*wLinesCount = TotalXferLines;
	g_isScanning=FALSE;

	XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine: leave MustScanner_GetRgb48BitLine");
	return TRUE;
}

/**********************************************************************
Author: Jack             Date: 2005/05/15
Routine Description: 
	Repair line when double CCD and color is 48bit
Parameters:
	lpLine: point to image be repaired
	isOrderInvert: RGB or BGR
	wLinesCount: how many line be repaired
Return value: 
	if the operation is success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL MustScanner_GetRgb48BitLine1200DPI(BYTE* lpLine,BOOL isOrderInvert,WORD *wLinesCount)
{
	XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine1200DPI: call in ");

	WORD wWantedTotalLines;
	WORD TotalXferLines;   
	
	WORD wRLinePosOdd=0;
	WORD wGLinePosOdd=0;
	WORD wBLinePosOdd=0;
	WORD wRLinePosEven=0;
	WORD wGLinePosEven=0;
	WORD wBLinePosEven=0;	
	DWORD wRTempData;
	DWORD wGTempData;
	DWORD wBTempData;
  DWORD wNextTempData;
	WORD i;

	TotalXferLines=0;
	wWantedTotalLines=*wLinesCount;

	g_isCanceled=FALSE;
	g_isScanning=TRUE;

	if(g_bFirstReadImage)
	{
		pthread_create(&g_threadid_readimage,NULL,MustScanner_ReadDataFromScanner,NULL);
		XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine1200DPI: thread create");
		g_bFirstReadImage=FALSE;
	}

	if(!isOrderInvert)
	{
		for ( ; TotalXferLines<wWantedTotalLines; ) 
		{
			if(g_dwTotalTotalXferLines>=g_SWHeight)
			{
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine1200DPI: thread exit");

				*wLinesCount = TotalXferLines;
				g_isScanning=FALSE;
				return TRUE;
			}

			if(GetScannedLines() > g_wtheReadyLines)
			{
        if(ST_Reflective == g_ScanType)
        {
				  wRLinePosOdd=(g_wtheReadyLines                  -g_wPixelDistance)%g_wMaxScanLines;
				  wGLinePosOdd=(g_wtheReadyLines-g_wLineDistance  -g_wPixelDistance)%g_wMaxScanLines;
				  wBLinePosOdd=(g_wtheReadyLines-g_wLineDistance*2-g_wPixelDistance)%g_wMaxScanLines;
				  wRLinePosEven=(g_wtheReadyLines                  )%g_wMaxScanLines;
				  wGLinePosEven=(g_wtheReadyLines-g_wLineDistance  )%g_wMaxScanLines;
				  wBLinePosEven=(g_wtheReadyLines-g_wLineDistance*2)%g_wMaxScanLines;
        }
        else
        {
				  wRLinePosEven=(g_wtheReadyLines                  -g_wPixelDistance)%g_wMaxScanLines;
				  wGLinePosEven=(g_wtheReadyLines-g_wLineDistance  -g_wPixelDistance)%g_wMaxScanLines;
				  wBLinePosEven=(g_wtheReadyLines-g_wLineDistance*2-g_wPixelDistance)%g_wMaxScanLines;
				  wRLinePosOdd=(g_wtheReadyLines                  )%g_wMaxScanLines;
				  wGLinePosOdd=(g_wtheReadyLines-g_wLineDistance  )%g_wMaxScanLines;
				  wBLinePosOdd=(g_wtheReadyLines-g_wLineDistance*2)%g_wMaxScanLines;          
        }
				
				for(i = 0; i < g_SWWidth; )
				{
					if(i + 1 != g_SWWidth) 
					{
						wRTempData =*(g_lpReadImageHead+wRLinePosOdd*g_BytesPerRow+i*6+0);		//R-channel
						wRTempData+=*(g_lpReadImageHead+wRLinePosOdd*g_BytesPerRow+i*6+1)<<8;	//R-channel
          wNextTempData =*(g_lpReadImageHead+wRLinePosEven*g_BytesPerRow+(i+1)*6+0);		//R-channel
						wNextTempData+=*(g_lpReadImageHead+wRLinePosEven*g_BytesPerRow+(i+1)*6+1)<<8;	//R-channel
          wRTempData = (wRTempData + wNextTempData) >> 1;
          
						wGTempData =*(g_lpReadImageHead+wGLinePosOdd*g_BytesPerRow+i*6+2);		//G-channel
						wGTempData+=*(g_lpReadImageHead+wGLinePosOdd*g_BytesPerRow+i*6+3)<<8;	//G-channel
          wNextTempData =*(g_lpReadImageHead+wGLinePosEven*g_BytesPerRow+(i+1)*6+2);		//G-channel
						wNextTempData+=*(g_lpReadImageHead+wGLinePosEven*g_BytesPerRow+(i+1)*6+3)<<8;	//G-channel
          wGTempData = (wGTempData + wNextTempData) >> 1;
            
						wBTempData =*(g_lpReadImageHead+wBLinePosOdd*g_BytesPerRow+i*6+4);		//B-channel
						wBTempData+=*(g_lpReadImageHead+wBLinePosOdd*g_BytesPerRow+i*6+5)<<8;	//B-channel
						wNextTempData =*(g_lpReadImageHead+wBLinePosEven*g_BytesPerRow+(i+1)*6+4);		//B-channel
						wNextTempData+=*(g_lpReadImageHead+wBLinePosEven*g_BytesPerRow+(i+1)*6+5)<<8;	//B-channel
           wBTempData = (wBTempData + wNextTempData) >> 1;
            
						*(lpLine+i*6+0)=LOBYTE(g_pGammaTable[wRTempData]);
						*(lpLine+i*6+1)=HIBYTE(g_pGammaTable[wRTempData]);
						*(lpLine+i*6+2)=LOBYTE(g_pGammaTable[wGTempData+65536]);
						*(lpLine+i*6+3)=HIBYTE(g_pGammaTable[wGTempData+65536]);
						*(lpLine+i*6+4)=LOBYTE(g_pGammaTable[wBTempData+131072]);
						*(lpLine+i*6+5)=HIBYTE(g_pGammaTable[wBTempData+131072]);
						i++;
						if (i >= g_SWWidth) 
						{
							break;
						}
						
						wRTempData =*(g_lpReadImageHead+wRLinePosEven*g_BytesPerRow+i*6+0);		//R-channel
						wRTempData+=*(g_lpReadImageHead+wRLinePosEven*g_BytesPerRow+i*6+1)<<8;	//R-channel
          wNextTempData =*(g_lpReadImageHead+wRLinePosOdd*g_BytesPerRow+(i+1)*6+0);		//R-channel
						wNextTempData+=*(g_lpReadImageHead+wRLinePosOdd*g_BytesPerRow+(i+1)*6+1)<<8;	//R-channel
          wRTempData = (wRTempData + wNextTempData) >> 1;
                      
						wGTempData =*(g_lpReadImageHead+wGLinePosEven*g_BytesPerRow+i*6+2);		//G-channel
						wGTempData+=*(g_lpReadImageHead+wGLinePosEven*g_BytesPerRow+i*6+3)<<8;	//G-channel
						wNextTempData =*(g_lpReadImageHead+wGLinePosOdd*g_BytesPerRow+(i+1)*6+2);		//G-channel
						wNextTempData+=*(g_lpReadImageHead+wGLinePosOdd*g_BytesPerRow+(i+1)*6+3)<<8;	//G-channel
           wGTempData = (wGTempData + wNextTempData) >> 1; 
                        
						wBTempData =*(g_lpReadImageHead+wBLinePosEven*g_BytesPerRow+i*6+4);		//B-channel
						wBTempData+=*(g_lpReadImageHead+wBLinePosEven*g_BytesPerRow+i*6+5)<<8;	//B-channel
						wNextTempData =*(g_lpReadImageHead+wBLinePosOdd*g_BytesPerRow+(i+1)*6+4);		//B-channel
						wNextTempData+=*(g_lpReadImageHead+wBLinePosOdd*g_BytesPerRow+(i+1)*6+5)<<8;	//B-channel
           wBTempData = (wBTempData + wNextTempData) >> 1; 
                        
						*(lpLine+i*6+0)=LOBYTE(g_pGammaTable[wRTempData]);
						*(lpLine+i*6+1)=HIBYTE(g_pGammaTable[wRTempData]);
						*(lpLine+i*6+2)=LOBYTE(g_pGammaTable[wGTempData+65536]);
						*(lpLine+i*6+3)=HIBYTE(g_pGammaTable[wGTempData+65536]);
						*(lpLine+i*6+4)=LOBYTE(g_pGammaTable[wBTempData+131072]);
						*(lpLine+i*6+5)=HIBYTE(g_pGammaTable[wBTempData+131072]);
            
/*            if(i + 1 == g_SWWidth)
            {
						    *(lpLine+i*6+0)=*(lpLine+(i-1)*6+0);
						    *(lpLine+i*6+1)=*(lpLine+(i-1)*6+1);
						    *(lpLine+i*6+2)=*(lpLine+(i-1)*6+2);
						    *(lpLine+i*6+3)=*(lpLine+(i-1)*6+3);
						    *(lpLine+i*6+4)=*(lpLine+(i-1)*6+4);
						    *(lpLine+i*6+5)=*(lpLine+(i-1)*6+5);              
            }
*/
						i++;
					}
				}					
				
				TotalXferLines++;
				g_dwTotalTotalXferLines++;
//				lpLine+=g_BytesPerRow;
				lpLine+=g_SWBytesPerRow;
//				g_wtheReadyLines++;
				AddReadyLines();
			}
			if(g_isCanceled)
			{
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine1200DPI: thread exit");
				break;

			}
		}//end for


	} 
	else
	{
		for ( ; TotalXferLines<wWantedTotalLines; ) 
		{
			if(g_dwTotalTotalXferLines>=g_SWHeight)
			{
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine1200DPI: thread exit");

				*wLinesCount = TotalXferLines;
				g_isScanning=FALSE;
				return TRUE;
			}

			if(GetScannedLines() > g_wtheReadyLines)
			{
        if(ST_Reflective == g_ScanType)
        {
				  wRLinePosOdd=(g_wtheReadyLines                  -g_wPixelDistance)%g_wMaxScanLines;
				  wGLinePosOdd=(g_wtheReadyLines-g_wLineDistance  -g_wPixelDistance)%g_wMaxScanLines;
				  wBLinePosOdd=(g_wtheReadyLines-g_wLineDistance*2-g_wPixelDistance)%g_wMaxScanLines;
				  wRLinePosEven=(g_wtheReadyLines                  )%g_wMaxScanLines;
				  wGLinePosEven=(g_wtheReadyLines-g_wLineDistance  )%g_wMaxScanLines;
				  wBLinePosEven=(g_wtheReadyLines-g_wLineDistance*2)%g_wMaxScanLines;
        }
        else
        {
				  wRLinePosEven=(g_wtheReadyLines                  -g_wPixelDistance)%g_wMaxScanLines;
				  wGLinePosEven=(g_wtheReadyLines-g_wLineDistance  -g_wPixelDistance)%g_wMaxScanLines;
				  wBLinePosEven=(g_wtheReadyLines-g_wLineDistance*2-g_wPixelDistance)%g_wMaxScanLines;
				  wRLinePosOdd=(g_wtheReadyLines                  )%g_wMaxScanLines;
				  wGLinePosOdd=(g_wtheReadyLines-g_wLineDistance  )%g_wMaxScanLines;
				  wBLinePosOdd=(g_wtheReadyLines-g_wLineDistance*2)%g_wMaxScanLines;          
        }
				
				for(i=0; i<g_SWWidth; )
				{
					if((i+1)!=g_SWWidth) 
					{
						wRTempData =*(g_lpReadImageHead+wRLinePosOdd*g_BytesPerRow+i*6+0); //R-channel
						wRTempData+=*(g_lpReadImageHead+wRLinePosOdd*g_BytesPerRow+i*6+1)<<8; //R-channel
          wNextTempData =*(g_lpReadImageHead+wRLinePosEven*g_BytesPerRow+(i+1)*6+0);		//R-channel
						wNextTempData+=*(g_lpReadImageHead+wRLinePosEven*g_BytesPerRow+(i+1)*6+1)<<8;	//R-channel
          wRTempData = (wRTempData + wNextTempData) >> 1;

						wGTempData =*(g_lpReadImageHead+wGLinePosOdd*g_BytesPerRow+i*6+2); //G-channel
						wGTempData+=*(g_lpReadImageHead+wGLinePosOdd*g_BytesPerRow+i*6+3)<<8; //G-channel
          wNextTempData =*(g_lpReadImageHead+wGLinePosEven*g_BytesPerRow+(i+1)*6+2);		//G-channel
						wNextTempData+=*(g_lpReadImageHead+wGLinePosEven*g_BytesPerRow+(i+1)*6+3)<<8;	//G-channel
          wGTempData = (wGTempData + wNextTempData) >> 1;
          
						wBTempData =*(g_lpReadImageHead+wBLinePosOdd*g_BytesPerRow+i*6+4); //B-channel
						wBTempData+=*(g_lpReadImageHead+wBLinePosOdd*g_BytesPerRow+i*6+5)<<8; //B-channel
          wNextTempData =*(g_lpReadImageHead+wBLinePosEven*g_BytesPerRow+(i+1)*6+4);		//B-channel
						wNextTempData+=*(g_lpReadImageHead+wBLinePosEven*g_BytesPerRow+(i+1)*6+5)<<8;	//B-channel
           wBTempData = (wBTempData + wNextTempData) >> 1;
           
						*(lpLine+i*6+4)=LOBYTE(g_pGammaTable[wRTempData]);
						*(lpLine+i*6+5)=HIBYTE(g_pGammaTable[wRTempData]);
						*(lpLine+i*6+2)=LOBYTE(g_pGammaTable[wGTempData+65536]);
						*(lpLine+i*6+3)=HIBYTE(g_pGammaTable[wGTempData+65536]);
						*(lpLine+i*6+0)=LOBYTE(g_pGammaTable[wBTempData+131072]);
						*(lpLine+i*6+1)=HIBYTE(g_pGammaTable[wBTempData+131072]);
						i++;
						if (i >= g_SWWidth) 
						{
							break;
						}
						
						wRTempData =*(g_lpReadImageHead+wRLinePosEven*g_BytesPerRow+i*6+0); //R-channel
						wRTempData+=*(g_lpReadImageHead+wRLinePosEven*g_BytesPerRow+i*6+1)<<8; //R-channel
          wNextTempData =*(g_lpReadImageHead+wRLinePosOdd*g_BytesPerRow+(i+1)*6+0);		//R-channel
						wNextTempData+=*(g_lpReadImageHead+wRLinePosOdd*g_BytesPerRow+(i+1)*6+1)<<8;	//R-channel
          wRTempData = (wRTempData + wNextTempData) >> 1;
                      
						wGTempData =*(g_lpReadImageHead+wGLinePosEven*g_BytesPerRow+i*6+2); //G-channel
						wGTempData+=*(g_lpReadImageHead+wGLinePosEven*g_BytesPerRow+i*6+3)<<8; //G-channel
 						wNextTempData =*(g_lpReadImageHead+wGLinePosOdd*g_BytesPerRow+(i+1)*6+2);		//G-channel
						wNextTempData+=*(g_lpReadImageHead+wGLinePosOdd*g_BytesPerRow+(i+1)*6+3)<<8;	//G-channel
           wGTempData = (wGTempData + wNextTempData) >> 1; 
            
						wBTempData =*(g_lpReadImageHead+wBLinePosEven*g_BytesPerRow+i*6+4); //B-channel
						wBTempData+=*(g_lpReadImageHead+wBLinePosEven*g_BytesPerRow+i*6+5)<<8; //B-channel
 						wNextTempData =*(g_lpReadImageHead+wBLinePosOdd*g_BytesPerRow+(i+1)*6+4);		//B-channel
						wNextTempData+=*(g_lpReadImageHead+wBLinePosOdd*g_BytesPerRow+(i+1)*6+5)<<8;	//B-channel
           wBTempData = (wBTempData + wNextTempData) >> 1; 
            
						*(lpLine+i*6+4)=LOBYTE(g_pGammaTable[wRTempData]);
						*(lpLine+i*6+5)=HIBYTE(g_pGammaTable[wRTempData]);
						*(lpLine+i*6+2)=LOBYTE(g_pGammaTable[wGTempData+65536]);
						*(lpLine+i*6+3)=HIBYTE(g_pGammaTable[wGTempData+65536]);
						*(lpLine+i*6+0)=LOBYTE(g_pGammaTable[wBTempData+131072]);
						*(lpLine+i*6+1)=HIBYTE(g_pGammaTable[wBTempData+131072]);
/*            
            if(i + 1 == g_SWWidth)
            {
						    *(lpLine+i*6+4)=*(lpLine+(i-1)*6+4);
						    *(lpLine+i*6+5)=*(lpLine+(i-1)*6+5);
						    *(lpLine+i*6+2)=*(lpLine+(i-1)*6+2);
						    *(lpLine+i*6+3)=*(lpLine+(i-1)*6+3);
						    *(lpLine+i*6+0)=*(lpLine+(i-1)*6+0);
						    *(lpLine+i*6+1)=*(lpLine+(i-1)*6+1);             
            }
*/
						i++;
					}
				}					
				
				TotalXferLines++;
				g_dwTotalTotalXferLines++;
//				lpLine+=g_BytesPerRow;
				lpLine+=g_SWBytesPerRow;
//				g_wtheReadyLines++;
				AddReadyLines();
			}
			if(g_isCanceled)
			{	
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
	
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine1200DPI: thread exit");

				break;
			}
		}//end for
	}
	
	*wLinesCount = TotalXferLines;
	g_isScanning=FALSE;

	XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine1200DPI: leave MustScanner_GetRgb48BitLine1200DPI");
	return TRUE;
}

/**********************************************************************
Author: Jack             Date: 2005/05/15
Routine Description: 
	Repair line when single CCD and color is 24bit
Parameters:
	lpLine: point to image be repaired
	isOrderInvert: RGB or BGR
	wLinesCount: how many line be repaired
Return value: 
	if the operation is success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL MustScanner_GetRgb24BitLine(BYTE* lpLine,BOOL isOrderInvert,WORD *wLinesCount)
{
	XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: call in");
	WORD wWantedTotalLines;
	WORD TotalXferLines;
	WORD wRLinePos=0;
	WORD wGLinePos=0;
	WORD wBLinePos=0;
	BYTE byRed;
	BYTE byGreen;
	BYTE byBlue;
	BYTE bNextPixel=0;
	WORD i;

	WORD tempR,tempG,tempB;
	
	g_isCanceled=FALSE;
	g_isScanning=TRUE;

	wWantedTotalLines=*wLinesCount;
	XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: get wWantedTotalLines= %d",wWantedTotalLines);

	TotalXferLines=0;
	
	if(g_bFirstReadImage)
	{
		pthread_create(&g_threadid_readimage,NULL,MustScanner_ReadDataFromScanner,NULL);
		XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: thread create");

		g_bFirstReadImage=FALSE;
	}	

	if(!isOrderInvert)
	{		
		XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: !isOrderInvert");

		for ( ; TotalXferLines<wWantedTotalLines; ) 
		{
			if(g_dwTotalTotalXferLines>=g_SWHeight)
			{
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: thread exit");

				*wLinesCount = TotalXferLines;
				g_isScanning=FALSE;
				return TRUE;
			}

			if(GetScannedLines() > g_wtheReadyLines)
			{				
				wRLinePos=g_wtheReadyLines%g_wMaxScanLines;
				wGLinePos=(g_wtheReadyLines-g_wLineDistance)%g_wMaxScanLines;
				wBLinePos=(g_wtheReadyLines-g_wLineDistance*2)%g_wMaxScanLines;
				
				for(i=0; i<g_SWWidth; i++)
				{
					byRed   =*(g_lpReadImageHead+wRLinePos*g_BytesPerRow+i*3+0);
					bNextPixel = *(g_lpReadImageHead+wRLinePos*g_BytesPerRow+(i+1)*3+0); //R-channel
					byRed = (byRed + bNextPixel) >> 1;
					
					byGreen =*(g_lpReadImageHead+wGLinePos*g_BytesPerRow+i*3+1);
					bNextPixel=*(g_lpReadImageHead+wGLinePos*g_BytesPerRow+(i+1)*3+1); //G-channel
					byGreen = (byGreen + bNextPixel) >> 1;
					
					byBlue  =*(g_lpReadImageHead+wBLinePos*g_BytesPerRow+i*3+2);
					bNextPixel=*(g_lpReadImageHead+wBLinePos*g_BytesPerRow+(i+1)*3+2); //B-channel
					byBlue = (byBlue + bNextPixel) >> 1;

#ifdef ENABLE_GAMMA
					tempR = (WORD)((byRed<<4)  | QBET4(byBlue,byGreen));
					tempG = (WORD)((byGreen<<4)| QBET4(byRed,byBlue));
					tempB = (WORD)((byBlue<<4) | QBET4(byGreen,byRed));   				    
					
					*(lpLine+i*3+0) = (unsigned char)(*(g_pGammaTable     +tempR));
					*(lpLine+i*3+1) = (unsigned char)(*(g_pGammaTable+4096+tempG));
					*(lpLine+i*3+2) = (unsigned char)(*(g_pGammaTable+8192+tempB));
#else
					*(lpLine+i*3+0) = (unsigned char)byRed;
					*(lpLine+i*3+1) = (unsigned char)byGreen;
					*(lpLine+i*3+2) = (unsigned char)byBlue;
#endif
				}					
								
				TotalXferLines++;
				g_dwTotalTotalXferLines++;
				lpLine+=g_SWBytesPerRow;
//				g_wtheReadyLines++;
				AddReadyLines();
					
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: g_dwTotalTotalXferLines=%d,g_SWHeight=%d",g_dwTotalTotalXferLines,g_SWHeight);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: g_SWBytesPerRow=%d",g_SWBytesPerRow);
			}
			if(g_isCanceled)
			{
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: thread exit");

				break;
			}
		} //end for
	}
	else
	{
		XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: isOrderInvert is TRUE");
		for ( ; TotalXferLines<wWantedTotalLines; ) 
		{
			if(g_dwTotalTotalXferLines>=g_SWHeight)
			{
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: thread exit");

				*wLinesCount = TotalXferLines;
				g_isScanning=FALSE;
				return TRUE;
			}

			if(GetScannedLines() > g_wtheReadyLines)
			{
				wRLinePos=g_wtheReadyLines%g_wMaxScanLines;
				wGLinePos=(g_wtheReadyLines-g_wLineDistance)%g_wMaxScanLines;
				wBLinePos=(g_wtheReadyLines-g_wLineDistance*2)%g_wMaxScanLines;
				
				for(i=0; i<g_SWWidth; i++)
				{
					XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: before byRed");
					byRed   =*(g_lpReadImageHead+wRLinePos*g_BytesPerRow+i*3+0);
					bNextPixel = *(g_lpReadImageHead+wRLinePos*g_BytesPerRow+(i+1)*3+0); //R-channel
					byRed = (byRed + bNextPixel) >> 1;
					
					XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: before byGreen");
					
					byGreen =*(g_lpReadImageHead+wGLinePos*g_BytesPerRow+i*3+1);
					bNextPixel=*(g_lpReadImageHead+wGLinePos*g_BytesPerRow+(i+1)*3+1); //G-channel
					byGreen = (byGreen + bNextPixel) >> 1;
					
					XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: before byBlue");
					
					byBlue  =*(g_lpReadImageHead+wBLinePos*g_BytesPerRow+i*3+2);
					bNextPixel=*(g_lpReadImageHead+wBLinePos*g_BytesPerRow+(i+1)*3+2); //B-channel
					byBlue = (byBlue + bNextPixel) >> 1;
					
					XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: before set lpLine");
					XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: i=%d",i);
#ifdef ENABLE_GAMMA
					*(lpLine+i*3+2) = (unsigned char)(*(g_pGammaTable    +(WORD)((byRed<<4)  | QBET4(byBlue,byGreen))));
					*(lpLine+i*3+1) = (unsigned char)(*(g_pGammaTable+4096+(WORD)((byGreen<<4)| QBET4(byRed,byBlue))));
					*(lpLine+i*3+0) = (unsigned char)(*(g_pGammaTable+8192+(WORD)((byBlue<<4) | QBET4(byGreen,byRed))));
#else
					*(lpLine+i*3+2) = (unsigned char)byRed;
					*(lpLine+i*3+1) = (unsigned char)byGreen;
					*(lpLine+i*3+0) = (unsigned char)byBlue;
#endif					
				}					
				
				TotalXferLines++;
				g_dwTotalTotalXferLines++;
				//lpLine+=g_BytesPerRow;
				lpLine+=g_SWBytesPerRow;
//				g_wtheReadyLines++;
				AddReadyLines();
				
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: g_dwTotalTotalXferLines=%d,g_SWHeight=%d",g_dwTotalTotalXferLines,g_SWHeight);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: g_SWBytesPerRow=%d",g_SWBytesPerRow);
			}
			if(g_isCanceled)
			{
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: thread exit");

				break;
			}
		} //end for
	}

	*wLinesCount = TotalXferLines;
	g_isScanning=FALSE;

	XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: leave MustScanner_GetRgb24BitLine");
	return TRUE;
}

/**********************************************************************
Author: Jack             Date: 2005/05/15
Routine Description: 
	Repair line when double CCD and color is 24bit
Parameters:
	lpLine: point to image be repaired
	isOrderInvert: RGB or BGR
	wLinesCount: how many line be repaired
Return value: 
	if the operation is success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL MustScanner_GetRgb24BitLine1200DPI(BYTE* lpLine,BOOL isOrderInvert,WORD *wLinesCount)
{
	XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1200DPI: call in");

	BYTE* lpTemp;
	WORD wWantedTotalLines;
	WORD TotalXferLines;
	WORD wRLinePosOdd=0;
	WORD wGLinePosOdd=0;
	WORD wBLinePosOdd=0;
	WORD wRLinePosEven=0;
	WORD wGLinePosEven=0;
	WORD wBLinePosEven=0;
	BYTE byRed;
	BYTE byGreen;
	BYTE byBlue;
	BYTE bNextPixel=0;
	WORD i;

	g_isCanceled=FALSE;
	g_isScanning=TRUE;
	TotalXferLines=0;
	wWantedTotalLines=*wLinesCount;
	lpTemp = lpLine;

	if(g_bFirstReadImage)
	{
		pthread_create(&g_threadid_readimage,NULL,MustScanner_ReadDataFromScanner,NULL);
		XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1200DPI: thread create");

		g_bFirstReadImage=FALSE;
	}

	if(!isOrderInvert)
	{
		for ( ; TotalXferLines<wWantedTotalLines; )
		{
			if(g_dwTotalTotalXferLines>=g_SWHeight)
			{
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1200DPI: g_dwTotalTotalXferLines=%d",g_dwTotalTotalXferLines);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1200DPI: g_Height=%d",g_Height);

				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1200DPI: thread exit");

				*wLinesCount = TotalXferLines;
				g_isScanning=FALSE;
				return TRUE;
			}

			if(GetScannedLines() > g_wtheReadyLines)
			{
        if(ST_Reflective == g_ScanType)
        {				
				  wRLinePosOdd=(g_wtheReadyLines                  -g_wPixelDistance)%g_wMaxScanLines;
				  wGLinePosOdd=(g_wtheReadyLines-g_wLineDistance  -g_wPixelDistance)%g_wMaxScanLines;
				  wBLinePosOdd=(g_wtheReadyLines-g_wLineDistance*2-g_wPixelDistance)%g_wMaxScanLines;
				  wRLinePosEven=(g_wtheReadyLines                  )%g_wMaxScanLines;
				  wGLinePosEven=(g_wtheReadyLines-g_wLineDistance  )%g_wMaxScanLines;
				  wBLinePosEven=(g_wtheReadyLines-g_wLineDistance*2)%g_wMaxScanLines;
        }
        else
        {
          wRLinePosEven=(g_wtheReadyLines                  -g_wPixelDistance)%g_wMaxScanLines;
				  wGLinePosEven=(g_wtheReadyLines-g_wLineDistance  -g_wPixelDistance)%g_wMaxScanLines;
				  wBLinePosEven=(g_wtheReadyLines-g_wLineDistance*2-g_wPixelDistance)%g_wMaxScanLines;
				  wRLinePosOdd=(g_wtheReadyLines                  )%g_wMaxScanLines;
				  wGLinePosOdd=(g_wtheReadyLines-g_wLineDistance  )%g_wMaxScanLines;
				  wBLinePosOdd=(g_wtheReadyLines-g_wLineDistance*2)%g_wMaxScanLines; 
        }
        
				
				for(i=0; i<g_SWWidth; )
				{
					if((i+1)!=g_SWWidth) 
					{
						byRed   =*(g_lpReadImageHead+wRLinePosOdd*g_BytesPerRow+i*3+0);
						bNextPixel=*(g_lpReadImageHead+wRLinePosEven*g_BytesPerRow+(i+1)*3+0); //R-channel
						byRed = (byRed + bNextPixel) >> 1;
						
						byGreen =*(g_lpReadImageHead+wGLinePosOdd*g_BytesPerRow+i*3+1);
						bNextPixel=*(g_lpReadImageHead+wGLinePosEven*g_BytesPerRow+(i+1)*3+1); //G-channel
						byGreen = (byGreen + bNextPixel) >> 1;
						
						byBlue  =*(g_lpReadImageHead+wBLinePosOdd*g_BytesPerRow+i*3+2);
						bNextPixel=*(g_lpReadImageHead+wBLinePosEven*g_BytesPerRow+(i+1)*3+2); //B-channel
						byBlue = (byBlue + bNextPixel) >> 1;
#ifdef ENABLE_GAMMA
						*(lpLine+i*3+0) = (unsigned char)(*(g_pGammaTable    +(WORD)((byRed<<4)  | QBET4(byBlue,byGreen))));
						*(lpLine+i*3+1) = (unsigned char)(*(g_pGammaTable+4096+(WORD)((byGreen<<4)| QBET4(byRed,byBlue))));
						*(lpLine+i*3+2) = (unsigned char)(*(g_pGammaTable+8192+(WORD)((byBlue<<4) | QBET4(byGreen,byRed))));
#else
						*(lpLine+i*3+0) = (unsigned char)byRed;
						*(lpLine+i*3+1) = (unsigned char)byGreen;
						*(lpLine+i*3+2) = (unsigned char)byBlue;
#endif

/*
          if(i + 2 >=  g_SWWidth)
          {
						  *(lpLine+i*3+0) = *(lpLine+(i-1)*3+0);
						  *(lpLine+i*3+1) = *(lpLine+(i-1)*3+1);
						  *(lpLine+i*3+2) = *(lpLine+(i-1)*3+2);               
          }
*/
						i++;
						if (i >= g_SWWidth) 
						{
							break;
						}

						byRed	=*(g_lpReadImageHead+wRLinePosEven*g_BytesPerRow+i*3+0);
						bNextPixel=*(g_lpReadImageHead+wRLinePosOdd*g_BytesPerRow+(i+1)*3+0); //R-channel
						byRed = (byRed + bNextPixel) >> 1;
						
						byGreen	=*(g_lpReadImageHead+wGLinePosEven*g_BytesPerRow+i*3+1);
						bNextPixel=*(g_lpReadImageHead+wGLinePosOdd*g_BytesPerRow+(i+1)*3+1); //G-channel
						byGreen = (byGreen + bNextPixel) >> 1;
						
						byBlue	=*(g_lpReadImageHead+wBLinePosEven*g_BytesPerRow+i*3+2);
						bNextPixel=*(g_lpReadImageHead+wBLinePosOdd*g_BytesPerRow+(i+1)*3+2); //B-channel
						byBlue = (byBlue + bNextPixel) >> 1;
#ifdef ENABLE_GAMMA
						*(lpLine+i*3+0) = (unsigned char)(*(g_pGammaTable    +(WORD)((byRed<<4)  | QBET4(byBlue,byGreen))));
						*(lpLine+i*3+1) = (unsigned char)(*(g_pGammaTable+4096+(WORD)((byGreen<<4)| QBET4(byRed,byBlue))));
						*(lpLine+i*3+2) = (unsigned char)(*(g_pGammaTable+8192+(WORD)((byBlue<<4) | QBET4(byGreen,byRed))));
#else
						*(lpLine+i*3+0) = (unsigned char)byRed;
						*(lpLine+i*3+1) = (unsigned char)byGreen;
						*(lpLine+i*3+2) = (unsigned char)byBlue;
#endif
/*
          if(i + 2 >= g_SWWidth)
          {
						  *(lpLine+i*3+0) = *(lpLine+(i-1)*3+0);
						  *(lpLine+i*3+1) = *(lpLine+(i-1)*3+1);
						  *(lpLine+i*3+2) = *(lpLine+(i-1)*3+2);            
          }
*/
						i++;
					}
				}					
				
				TotalXferLines++;
				g_dwTotalTotalXferLines++;
				lpLine+=g_SWBytesPerRow;
//				g_wtheReadyLines++;
				AddReadyLines();

				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1200DPI: g_dwTotalTotalXferLines=%d",g_dwTotalTotalXferLines);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1200DPI: g_Height=%d",g_Height);
			
			}
			if(g_isCanceled)
			{
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1200DPI: thread exit");

				break;
			}
					
		} //end for
	}
	else
	{
		for ( ; TotalXferLines<wWantedTotalLines; ) 
		{
			if(g_dwTotalTotalXferLines>=g_SWHeight)
			{
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1200DPI: g_dwTotalTotalXferLines=%d",g_dwTotalTotalXferLines);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1200DPI: g_Height=%d",g_Height);

				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1200DPI: thread exit");

				*wLinesCount = TotalXferLines;
				g_isScanning=FALSE;
				return TRUE;
			}

			if(GetScannedLines() > g_wtheReadyLines)
			{
        if(ST_Reflective == g_ScanType)
        {
				  wRLinePosOdd=(g_wtheReadyLines                  -g_wPixelDistance)%g_wMaxScanLines;
				  wGLinePosOdd=(g_wtheReadyLines-g_wLineDistance  -g_wPixelDistance)%g_wMaxScanLines;
				  wBLinePosOdd=(g_wtheReadyLines-g_wLineDistance*2-g_wPixelDistance)%g_wMaxScanLines;
				  wRLinePosEven=(g_wtheReadyLines                  )%g_wMaxScanLines;
				  wGLinePosEven=(g_wtheReadyLines-g_wLineDistance  )%g_wMaxScanLines;
				  wBLinePosEven=(g_wtheReadyLines-g_wLineDistance*2)%g_wMaxScanLines;          
        }
        else
        {
				  wRLinePosEven=(g_wtheReadyLines                  -g_wPixelDistance)%g_wMaxScanLines;
				  wGLinePosEven=(g_wtheReadyLines-g_wLineDistance  -g_wPixelDistance)%g_wMaxScanLines;
				  wBLinePosEven=(g_wtheReadyLines-g_wLineDistance*2-g_wPixelDistance)%g_wMaxScanLines;
				  wRLinePosOdd=(g_wtheReadyLines                  )%g_wMaxScanLines;
				  wGLinePosOdd=(g_wtheReadyLines-g_wLineDistance  )%g_wMaxScanLines;
				  wBLinePosOdd=(g_wtheReadyLines-g_wLineDistance*2)%g_wMaxScanLines;
        }
				
				for(i=0; i<g_SWWidth; )
				{
					if((i+1)!=g_SWWidth) 
					{
						byRed   =*(g_lpReadImageHead+wRLinePosOdd*g_BytesPerRow+i*3+0);
						bNextPixel=*(g_lpReadImageHead+wRLinePosEven*g_BytesPerRow+(i+1)*3+0); //R-channel
						byRed = (byRed + bNextPixel) >> 1;
						
						byGreen =*(g_lpReadImageHead+wGLinePosOdd*g_BytesPerRow+i*3+1);
						bNextPixel=*(g_lpReadImageHead+wGLinePosEven*g_BytesPerRow+(i+1)*3+1); //G-channel
						byGreen = (byGreen + bNextPixel) >> 1;
						
						byBlue  =*(g_lpReadImageHead+wBLinePosOdd*g_BytesPerRow+i*3+2);
						bNextPixel=*(g_lpReadImageHead+wBLinePosEven*g_BytesPerRow+(i+1)*3+2); //B-channel
						byBlue = (byBlue + bNextPixel) >> 1;

#ifdef ENABLE_GAMMA
						*(lpLine+i*3+2) = (unsigned char)(*(g_pGammaTable    +(WORD)((byRed<<4)  | QBET4(byBlue,byGreen))));
						*(lpLine+i*3+1) = (unsigned char)(*(g_pGammaTable+4096+(WORD)((byGreen<<4)| QBET4(byRed,byBlue))));
						*(lpLine+i*3+0) = (unsigned char)(*(g_pGammaTable+8192+(WORD)((byBlue<<4) | QBET4(byGreen,byRed))));
#else
						*(lpLine+i*3+2) = (unsigned char)byRed;
						*(lpLine+i*3+1) = (unsigned char)byGreen;
						*(lpLine+i*3+0) = (unsigned char)byBlue;
#endif
						i++;
						if (i >= g_SWWidth) 
						{
							break;
						}
						
						byRed	=*(g_lpReadImageHead+wRLinePosEven*g_BytesPerRow+i*3+0);
						bNextPixel=*(g_lpReadImageHead+wRLinePosOdd*g_BytesPerRow+(i+1)*3+0); //R-channel
						byRed = (byRed + bNextPixel) >> 1;
						
						byGreen	=*(g_lpReadImageHead+wGLinePosEven*g_BytesPerRow+i*3+1);
						bNextPixel=*(g_lpReadImageHead+wGLinePosOdd*g_BytesPerRow+(i+1)*3+1); //G-channel
						byGreen = (byGreen + bNextPixel) >> 1;
						
						byBlue	=*(g_lpReadImageHead+wBLinePosEven*g_BytesPerRow+i*3+2);
						bNextPixel=*(g_lpReadImageHead+wBLinePosOdd*g_BytesPerRow+(i+1)*3+2); //B-channel
						byBlue = (byBlue + bNextPixel) >> 1;
#ifdef ENABLE_GAMMA
						*(lpLine+i*3+2) = (unsigned char)(*(g_pGammaTable    +(WORD)((byRed<<4)  | QBET4(byBlue,byGreen))));
						*(lpLine+i*3+1) = (unsigned char)(*(g_pGammaTable+4096+(WORD)((byGreen<<4)| QBET4(byRed,byBlue))));
						*(lpLine+i*3+0) = (unsigned char)(*(g_pGammaTable+8192+(WORD)((byBlue<<4) | QBET4(byGreen,byRed))));
#else
						*(lpLine+i*3+2) = (unsigned char)byRed;
						*(lpLine+i*3+1) = (unsigned char)byGreen;
						*(lpLine+i*3+0) = (unsigned char)byBlue;
#endif
/*
          if(i + 1 == g_SWWidth)
          {
						  *(lpLine+i*3+2) = *(lpLine+(i-1)*3+2);
						  *(lpLine+i*3+1) = *(lpLine+(i-1)*3+1);
						  *(lpLine+i*3+0) = *(lpLine+(i-1)*3+0);
          }
*/
						i++;
					}
				}					
				
				TotalXferLines++;
				g_dwTotalTotalXferLines++;
//				lpLine+=g_BytesPerRow;
				lpLine+=g_SWBytesPerRow;
//				g_wtheReadyLines++;
				AddReadyLines();

				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1200DPI: g_dwTotalTotalXferLines=%d",g_dwTotalTotalXferLines);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1200DPI: g_Height=%d",g_Height);

			}
			if(g_isCanceled)
			{
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1200DPI: thread exit");


				break;
			}
		} //end for
	}		
	
	if(!g_isCanceled)
	{
		if(g_bSharpen)
			MustScanner_CrossSharpen(lpTemp,wWantedTotalLines,g_SWBytesPerRow,2);
	}
	
	*wLinesCount = TotalXferLines;
	g_isScanning=FALSE;

	XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1200DPI: leave MustScanner_GetRgb24BitLine1200DPI");
	return TRUE;
}

/**********************************************************************
Author: Jack             Date: 2005/05/15
Routine Description: 
	Repair line when single CCD and color is 16bit
Parameters:
	lpLine: point to image be repaired
	isOrderInvert: RGB or BGR
	wLinesCount: how many line be repaired
Return value: 
	if the operation is success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL MustScanner_GetMono16BitLine(BYTE* lpLine,BOOL isOrderInvert,WORD *wLinesCount)
{
	XDBG(DEBUG_SPICALL,"MustScanner_GetMono16BitLine: call in");
	WORD wWantedTotalLines;
	WORD TotalXferLines;   
	DWORD wTempData;
	
	WORD wLinePos=0;
	WORD i;

	TotalXferLines=0;
	g_isCanceled=FALSE;
	g_isScanning=TRUE;
	wWantedTotalLines=*wLinesCount;	
	
	if(g_bFirstReadImage)
	{
		pthread_create(&g_threadid_readimage,NULL,MustScanner_ReadDataFromScanner,NULL);
		XDBG(DEBUG_SPICALL,"MustScanner_GetMono16BitLine: thread create");
		g_bFirstReadImage=FALSE;
	}
	
	for ( ; TotalXferLines<wWantedTotalLines; ) 
	{
		
		if(g_dwTotalTotalXferLines>=g_SWHeight)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono16BitLine: thread exit");

			*wLinesCount = TotalXferLines;
			g_isScanning=FALSE;
			return TRUE;
		}

		if(GetScannedLines() > g_wtheReadyLines)
		{
			wLinePos=g_wtheReadyLines%g_wMaxScanLines;
			
			for(i=0; i<g_SWWidth; i++)
			{
				wTempData =*(g_lpReadImageHead+wLinePos*g_BytesPerRow+i*2+0);
				wTempData+=*(g_lpReadImageHead+wLinePos*g_BytesPerRow+i*2+1)<<8;
				*(lpLine+i*2+0)=LOBYTE(g_pGammaTable[wTempData]);
				*(lpLine+i*2+1)=HIBYTE(g_pGammaTable[wTempData]);
			}					
			
			TotalXferLines++;
			g_dwTotalTotalXferLines++;
//			lpLine+=g_BytesPerRow;
			lpLine+=g_SWBytesPerRow;
//			g_wtheReadyLines++;
			AddReadyLines();
		} 
		if(g_isCanceled)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono16BitLine: thread exit");

			break;
		}
	}//end for	

	*wLinesCount = TotalXferLines;
	g_isScanning=FALSE;

	XDBG(DEBUG_SPICALL,"MustScanner_GetMono16BitLine: leave MustScanner_GetMono16BitLine");
	return TRUE;
}

/**********************************************************************
Author: Jack             Date: 2005/05/15
Routine Description: 
	Repair line when double CCD and color is 16bit
Parameters:
	lpLine: point to image be repaired
	isOrderInvert: RGB or BGR
	wLinesCount: how many line be repaired
Return value: 
	if the operation is success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL MustScanner_GetMono16BitLine1200DPI(BYTE* lpLine,BOOL isOrderInvert,WORD *wLinesCount)
{
	XDBG(DEBUG_SPICALL,"MustScanner_GetMono16BitLine1200DPI: call in");
	WORD wWantedTotalLines;
	WORD TotalXferLines;   
	DWORD dwTempData;
	WORD wLinePosOdd=0;
	WORD wLinePosEven=0;	
	WORD i;

	TotalXferLines=0;
	g_isCanceled=FALSE;
	g_isScanning=TRUE;
	wWantedTotalLines=*wLinesCount;
	
	if(g_bFirstReadImage)
	{
		pthread_create(&g_threadid_readimage,NULL,MustScanner_ReadDataFromScanner,NULL);
		XDBG(DEBUG_SPICALL,"MustScanner_GetMono16BitLine1200DPI: thread create");
		g_bFirstReadImage=FALSE;
	}

	for ( ; TotalXferLines<wWantedTotalLines; ) 
	{
		if(g_dwTotalTotalXferLines>=g_SWHeight)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono16BitLine1200DPI: thread exit");

			*wLinesCount = TotalXferLines;
			g_isScanning=FALSE;
			return TRUE;
		}

		if(GetScannedLines() > g_wtheReadyLines)
		{
      if(ST_Reflective == g_ScanType)
      {
        wLinePosOdd=(g_wtheReadyLines-g_wPixelDistance)%g_wMaxScanLines;
			  wLinePosEven=(g_wtheReadyLines)%g_wMaxScanLines;       
      }
      else
      {
  			wLinePosEven=(g_wtheReadyLines-g_wPixelDistance)%g_wMaxScanLines;
			  wLinePosOdd=(g_wtheReadyLines)%g_wMaxScanLines;        
      }

			
			for(i=0; i<g_SWWidth; )
			{
				if((i+1)!=g_SWWidth) 
				{
					dwTempData =(DWORD)(*(g_lpReadImageHead+wLinePosOdd*g_BytesPerRow+i*2+0)); 
					dwTempData+=(DWORD)(*(g_lpReadImageHead+wLinePosOdd*g_BytesPerRow+i*2+1)<<8); 
					dwTempData+=(DWORD)(*(g_lpReadImageHead+wLinePosEven*g_BytesPerRow+(i+1)*2+0)); 
					dwTempData+=(DWORD)(*(g_lpReadImageHead+wLinePosEven*g_BytesPerRow+(i+1)*2+1)<<8); 
					dwTempData=g_pGammaTable[dwTempData>>1];
					*(lpLine+i*2+0)=LOBYTE((WORD)dwTempData);
					*(lpLine+i*2+1)=HIBYTE((WORD)dwTempData);
					i++;
					if (i >= g_SWWidth) 
					{
						break;
					}
					
					dwTempData =(DWORD)(*(g_lpReadImageHead+wLinePosEven*g_BytesPerRow+i*2+0)); 
					dwTempData+=(DWORD)(*(g_lpReadImageHead+wLinePosEven*g_BytesPerRow+i*2+1)<<8); 
					dwTempData+=(DWORD)(*(g_lpReadImageHead+wLinePosOdd*g_BytesPerRow+(i+1)*2+0)); 
					dwTempData+=(DWORD)(*(g_lpReadImageHead+wLinePosOdd*g_BytesPerRow+(i+1)*2+1)<<8); 
					dwTempData=g_pGammaTable[dwTempData>>1];
					*(lpLine+i*2+0)=LOBYTE((WORD)dwTempData);
					*(lpLine+i*2+1)=HIBYTE((WORD)dwTempData);
					i++;
				}
			}					
			
			TotalXferLines++;
			g_dwTotalTotalXferLines++;
//			lpLine+=g_BytesPerRow;
			lpLine+=g_SWBytesPerRow;
//			g_wtheReadyLines++;
			AddReadyLines();
		} 
		if(g_isCanceled)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono16BitLine1200DPI: thread exit");

			break;
		}
	}//end for
	
	*wLinesCount = TotalXferLines;
	g_isScanning=FALSE;

	XDBG(DEBUG_SPICALL,"MustScanner_GetMono16BitLine1200DPI: leave MustScanner_GetMono16BitLine1200DPI");
	return TRUE;
}

/**********************************************************************
Author: Jack             Date: 2005/05/15
Routine Description: 
	Repair line when single CCD and color is 8bit
Parameters:
	lpLine: point to image be repaired
	isOrderInvert: RGB or BGR
	wLinesCount: how many line be repaired
Return value: 
	if the operation is success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL MustScanner_GetMono8BitLine(BYTE* lpLine,BOOL isOrderInvert,WORD *wLinesCount)
{
	XDBG(DEBUG_SPICALL,"MustScanner_GetMono8BitLine: call in");
	WORD wWantedTotalLines;
	WORD TotalXferLines;   
	WORD i;
	WORD wLinePos=0;

	TotalXferLines=0;
	g_isCanceled=FALSE;
	g_isScanning=TRUE;
	wWantedTotalLines=*wLinesCount;

	if(g_bFirstReadImage)
	{
		pthread_create(&g_threadid_readimage,NULL,MustScanner_ReadDataFromScanner,NULL);
		XDBG(DEBUG_SPICALL,"MustScanner_GetMono8BitLine: thread create");
		g_bFirstReadImage=FALSE;
	}	
	
	for ( ; TotalXferLines<wWantedTotalLines; ) 
	{
		if(g_dwTotalTotalXferLines>=g_SWHeight)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono8BitLine: thread exit");

			*wLinesCount = TotalXferLines;
			g_isScanning=FALSE;
			return TRUE;
		}

		if(GetScannedLines() > g_wtheReadyLines)
		{
			wLinePos=g_wtheReadyLines%g_wMaxScanLines;
			
			for(i=0; i<g_SWWidth; i++)
			{
				*(lpLine+i) = (BYTE)*(g_pGammaTable+(WORD)((*(g_lpReadImageHead+wLinePos*g_BytesPerRow+i)<<4) 
					| (rand()&0x0f)));
			}				
			
			TotalXferLines++;
			g_dwTotalTotalXferLines++;
			lpLine+=g_SWBytesPerRow;
//			g_wtheReadyLines++;
			AddReadyLines();
			
		} 
		if(g_isCanceled)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono8BitLine: thread exit");

			break;
		}
	}//end for
	
	*wLinesCount = TotalXferLines;
	g_isScanning=FALSE;

	XDBG(DEBUG_SPICALL,"MustScanner_GetMono8BitLine: leave MustScanner_GetMono8BitLine");
	return TRUE;
}

/**********************************************************************
Author: Jack             Date: 2005/05/15
Routine Description: 
	Repair line when double CCD and color is 8bit
Parameters:
	lpLine: point to image be repaired
	isOrderInvert: RGB or BGR
	wLinesCount: how many line be repaired
Return value: 
	if the operation is success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL MustScanner_GetMono8BitLine1200DPI(BYTE* lpLine,BOOL isOrderInvert,WORD *wLinesCount)
{
	XDBG(DEBUG_SPICALL,"MustScanner_GetMono8BitLine1200DPI: call in");
	BYTE* lpTemp;		
	WORD wWantedTotalLines;
	WORD TotalXferLines;  
	
	WORD wLinePosOdd=0;
	WORD wLinePosEven=0;
	BYTE byGray;	
    WORD i;
	BYTE bNextPixel = 0;	

	TotalXferLines=0;
	g_isCanceled=FALSE;
	g_isScanning=TRUE;
	wWantedTotalLines=*wLinesCount;
	lpTemp=lpLine;

	if(g_bFirstReadImage)
	{
		pthread_create(&g_threadid_readimage,NULL,MustScanner_ReadDataFromScanner,NULL);
		XDBG(DEBUG_SPICALL,"MustScanner_GetMono8BitLine1200DPI: thread create");
		g_bFirstReadImage=FALSE;
	}

	for (;TotalXferLines<wWantedTotalLines;) 
	{
		//jason added height check
		if(g_dwTotalTotalXferLines>=g_SWHeight)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono8BitLine1200DPI: thread exit");

			*wLinesCount = TotalXferLines;
			g_isScanning=FALSE;
			return TRUE;
		}

		if(GetScannedLines() > g_wtheReadyLines)
		{
      if(ST_Reflective == g_ScanType)
      {
 			  wLinePosOdd=(g_wtheReadyLines - g_wPixelDistance)%g_wMaxScanLines;
			  wLinePosEven=(g_wtheReadyLines)%g_wMaxScanLines;       
      }
      else
      {
          wLinePosEven=(g_wtheReadyLines - g_wPixelDistance)%g_wMaxScanLines;
			    wLinePosOdd=(g_wtheReadyLines)%g_wMaxScanLines;
      }

			
			for(i=0; i<g_SWWidth; )
			{
				if((i+1)!=g_SWWidth) 
				{
					byGray=*(g_lpReadImageHead+wLinePosOdd*g_BytesPerRow+i);
					bNextPixel=*(g_lpReadImageHead+wLinePosEven*g_BytesPerRow+(i+1)); 
					byGray = (byGray + bNextPixel) >> 1;
					
					*(lpLine+i) = (BYTE)*(g_pGammaTable+( byGray<<4 | (rand()&0x0f)));
					i++;
					if (i >= g_SWWidth) 
					{
						break;
					}
					
					byGray=*(g_lpReadImageHead+wLinePosEven*g_BytesPerRow+i);
					bNextPixel=*(g_lpReadImageHead+wLinePosOdd*g_BytesPerRow+(i+1)); 
					byGray = (byGray + bNextPixel) >> 1;
					
					*(lpLine+i) = (BYTE)*(g_pGammaTable+( byGray<<4 | (rand()&0x0f)));
					i++;
				}
			}				
			
			TotalXferLines++;
			g_dwTotalTotalXferLines++;
			lpLine+=g_SWBytesPerRow;
//			g_wtheReadyLines++;
			AddReadyLines();
		}
		if(g_isCanceled)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono8BitLine1200DPI: thread exit");

			break;
		}
	}//end for
	
	if(!g_isCanceled)
	{
		if(g_bSharpen)
			MustScanner_CrossSharpen(lpTemp,wWantedTotalLines,g_SWBytesPerRow,1);
	}
  
	*wLinesCount = TotalXferLines;
	g_isScanning=FALSE;

	XDBG(DEBUG_SPICALL,"MustScanner_GetMono8BitLine1200DPI: leave MustScanner_GetMono8BitLine1200DPI");
	return TRUE;
}

/**********************************************************************
Author: Jack             Date: 2005/05/15
Routine Description: 
	Repair line when single CCD and color is 1bit
Parameters:
	lpLine: point to image be repaired
	isOrderInvert: RGB or BGR
	wLinesCount: how many line be repaired
Return value: 
	if the operation is success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL MustScanner_GetMono1BitLine(BYTE* lpLine,BOOL isOrderInvert,WORD *wLinesCount)
{
	XDBG(DEBUG_SPICALL,"MustScanner_GetMono1BitLine: call in");
	WORD wWantedTotalLines;
	WORD TotalXferLines;  
	WORD wLinePos;
	WORD i;

	g_isCanceled=FALSE;
	g_isScanning=TRUE;
	wWantedTotalLines=*wLinesCount;
	
	if(g_bFirstReadImage)
	{
		pthread_create(&g_threadid_readimage,NULL,MustScanner_ReadDataFromScanner,NULL);
		XDBG(DEBUG_SPICALL,"MustScanner_GetMono1BitLine: thread create");
		g_bFirstReadImage=FALSE;
	}
	
	memset(lpLine,0,wWantedTotalLines*g_SWWidth/8);
	
	for ( TotalXferLines=0; TotalXferLines<wWantedTotalLines; ) 

	{
		if(g_dwTotalTotalXferLines>=g_SWHeight)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono1BitLine: thread exit");

			*wLinesCount = TotalXferLines;
			g_isScanning=FALSE;
			return TRUE;
		}

		if(GetScannedLines() > g_wtheReadyLines)
		{
			wLinePos=g_wtheReadyLines%g_wMaxScanLines;
			
			for(i=0; i<g_SWWidth; i++ )
			{
				if(*(g_lpReadImageHead+wLinePos*g_BytesPerRow+i ) > g_wLineartThreshold) 
				{
					*(lpLine+i/8)+=(0x80 >> (i%8));
				}
			}
		
			TotalXferLines++;
			g_dwTotalTotalXferLines++;
//			lpLine+=(g_BytesPerRow/8);
			lpLine+=(g_SWBytesPerRow/8);
//			g_wtheReadyLines++;
			AddReadyLines();
		} 
		if(g_isCanceled)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono1BitLine: thread exit");

			break;
		}
	}//end for

	*wLinesCount = TotalXferLines;
	g_isScanning=FALSE;
	
	XDBG(DEBUG_SPICALL,"MustScanner_GetMono1BitLine: leave MustScanner_GetMono1BitLine");
	return TRUE;
}

/**********************************************************************
Author: Jack             Date: 2005/05/15
Routine Description: 
	Repair line when double CCD and color is 1bit
Parameters:
	lpLine: point to image be repaired
	isOrderInvert: RGB or BGR
	wLinesCount: how many line be repaired
Return value: 
	if the operation is success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL MustScanner_GetMono1BitLine1200DPI(BYTE* lpLine,BOOL isOrderInvert,WORD *wLinesCount)
{
	XDBG(DEBUG_SPICALL,"MustScanner_GetMono1BitLine1200DPI: call in");
	WORD wWantedTotalLines;
	WORD TotalXferLines;   
	WORD i;	
	WORD wLinePosOdd;
	WORD wLinePosEven;

	g_isCanceled=FALSE;
	g_isScanning=TRUE;
	wWantedTotalLines=*wLinesCount;
	
	if(g_bFirstReadImage)
	{
		pthread_create(&g_threadid_readimage,NULL,MustScanner_ReadDataFromScanner,NULL);
		XDBG(DEBUG_SPICALL,"MustScanner_GetMono1BitLine1200DPI: thread create");
		g_bFirstReadImage=FALSE;
	}
	
	memset(lpLine,0,wWantedTotalLines*g_SWWidth/8);
	
	for ( TotalXferLines=0; TotalXferLines<wWantedTotalLines; ) 
	{
		if(g_dwTotalTotalXferLines>=g_SWHeight)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono1BitLine1200DPI: thread exit");

			*wLinesCount = TotalXferLines;
			g_isScanning=FALSE;
			return TRUE;
		}
		
		if(GetScannedLines() > g_wtheReadyLines)
		{
      if(ST_Reflective == g_ScanType)
      {
 			  wLinePosEven=(g_wtheReadyLines  )%g_wMaxScanLines;
			  wLinePosOdd=(g_wtheReadyLines-g_wPixelDistance)%g_wMaxScanLines;       
      }
      else
      {
   			wLinePosOdd=(g_wtheReadyLines  )%g_wMaxScanLines;
			  wLinePosEven=(g_wtheReadyLines-g_wPixelDistance)%g_wMaxScanLines;
      }

			
			for(i=0; i<g_SWWidth; )
			{
				if((i+1)!=g_SWWidth) 
				{
					if(*(g_lpReadImageHead+wLinePosOdd*g_BytesPerRow+i) > g_wLineartThreshold) 
						*(lpLine+i/8)+=(0x80 >> (i%8)); 
					i++;
					if (i >= g_SWWidth) 
					{
						break;
					}

					if(*(g_lpReadImageHead+wLinePosEven*g_BytesPerRow+i) > g_wLineartThreshold) 
						*(lpLine+i/8)+=(0x80 >> (i%8)); 
					i++;
				}
			}					
			
			TotalXferLines++;
			g_dwTotalTotalXferLines++;
//			lpLine+=g_BytesPerRow/8;
			lpLine+=g_SWBytesPerRow/8;
//			g_wtheReadyLines++;
			AddReadyLines();
			
		} 
		if(g_isCanceled)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono1BitLine1200DPI: thread exit");

			break;
		}
	}//end for

	*wLinesCount = TotalXferLines;
	g_isScanning=FALSE;

	XDBG(DEBUG_SPICALL,"MustScanner_GetMono1BitLine1200DPI: leave MustScanner_GetMono1BitLine1200DPI");
	return TRUE;
}

/**********************************************************************
Author: Jack            Date: 2005/05/21
Routine Description: 
	prepare calculate Max and Min value	
Parameters:
	wResolution: the scan resolution
Return value:
	none 
***********************************************************************/
void MustScanner_PrepareCalculateMaxMin(WORD wResolution)
{
	g_wDarkCalWidth = 52; 
	if(wResolution <= 600)
	{
		g_wCalWidth = ((5120*wResolution/600+511)>>9)<<9;
		g_wDarkCalWidth = g_wDarkCalWidth / (1200 / wResolution); 
		
		if(wResolution < 200)
		{
			g_nPowerNum = 3;
			g_nSecLength = 8;// 2^nPowerNum
			g_nDarkSecLength = g_wDarkCalWidth / 2; // Dark has at least 2 sections
		}
		else
		{
			g_nPowerNum=6;
			g_nSecLength=64; // 2^nPowerNum
			g_nDarkSecLength = g_wDarkCalWidth / 3; 
		}
	}
	else
	{
		g_nPowerNum=6;
		g_nSecLength=64;  //2^nPowerNum
		g_wCalWidth=10240;
		g_nDarkSecLength = g_wDarkCalWidth / 5; 
	}
	
	if(g_nDarkSecLength <= 0)
	{
		g_nDarkSecLength = 1; 
	}
	
	g_wStartPosition = 13 * wResolution / 1200 ;
	g_wCalWidth -= g_wStartPosition;
		
	// start of find Max value
	g_nSecNum = (int)(g_wCalWidth/g_nSecLength);
	
	// start of fin min value
	g_nDarkSecNum = (int)(g_wDarkCalWidth/g_nDarkSecLength);  
}

/**********************************************************************
Author: Jack            Date: 2005/05/21
Routine Description: 
	calculate the Max and Min value
Parameters:
	pBuffer: the image data
	lpMaxValue: the max value
	lpMinValue: the min value
	wResolution: the scan resolution
Return value:
	none 
***********************************************************************/
void MustScanner_CalculateMaxMin(LPBYTE pBuffer, LPWORD lpMaxValue, LPWORD lpMinValue, WORD wResolution)
{
	WORD	*wSecData = NULL, *wDarkSecData = NULL;
	int		i,j;
	
	wSecData = (WORD*)malloc(sizeof(WORD)*g_nSecNum);
	if (wSecData == NULL)
	{
		return;
	}
	else
	{
		memset(wSecData, 0, g_nSecNum * sizeof(WORD));
	}
	
	for(i=0 ; i<g_nSecNum ; i++)
	{
		for (j=0 ;j<g_nSecLength ; j++)
			wSecData[i]+=*(pBuffer+g_wStartPosition+i*g_nSecLength+j);
		wSecData[i]>>=g_nPowerNum;
	}
	
	*lpMaxValue=wSecData[0];
	for(i=0; i<g_nSecNum; i++)
	{
		if(*lpMaxValue<wSecData[i])  
			*lpMaxValue=wSecData[i];
	}
	
	free(wSecData);
	
	wDarkSecData = (WORD*)malloc(sizeof(WORD) * g_nDarkSecNum);   
	if(wDarkSecData == NULL)
	{
		return;
	}
	else
	{
		memset(wDarkSecData, 0, g_nDarkSecNum * sizeof(WORD));
	}
	
	for(i=0 ; i<g_nDarkSecNum ; i++)
	{
		for (j=0 ;j<g_nDarkSecLength ; j++)
			wDarkSecData[i]+=*(pBuffer+g_wStartPosition+i*g_nDarkSecLength+j);
		
		wDarkSecData[i]/=g_nDarkSecLength;
	}
	
	*lpMinValue=wDarkSecData[0];
	for(i=0; i<g_nDarkSecNum; i++)
	{
		if(*lpMinValue>wDarkSecData[i])  
			*lpMinValue=wDarkSecData[i];
	}
	free(wDarkSecData);
}

/**********************************************************************
Author: Jack             Date: 2005/05/14
Routine Description: 
	Sharpen the image
Parameters:
	lpLine: the data of the image 
	wLineCount: the count of lines
	LineBytes: the bytes of per line
	ScanMode: the image scan mode
Return value: 
	none
***********************************************************************/
void MustScanner_CrossSharpen(BYTE* lpLine,WORD wLineCount, DWORD LineBytes, BYTE ScanMode)
{
	XDBG(DEBUG_SPICALL,"MustScanner_CrossSharpen: call in");

	USHORT Row,Col;
	USHORT Lines;
	UINT   Index;
	LPBYTE ImgAddr;
	short  TmpColor1,TmpColor2;
    float  NormalInfo[1020];
	int i;

	for(i=0;i<1020;i++)
	{			
		if(i<40)
		{
			NormalInfo[i] = (float)(0.5-0.0138); 
		}
		else if(i<80)
		{
			NormalInfo[i]= (float)(0.5-0.0712);  
		}
		else if(i<120)
		{
			NormalInfo[i]= (float)(0.5-0.1678);  
		}
		else if(i<160)
		{
			NormalInfo[i]= (float)(0.5-0.2397);  
		}
		else if(i<200)
		{

			NormalInfo[i]= (float)(0.5-0.2311);  
		}
		else if(i<240)
		{
			NormalInfo[i]= (float)(0.5-0.1585);  
		}
		else if(i<280)
		{
			NormalInfo[i]= (float)(0.5-0.0792);  
		}
		else if(i<320)
		{
			NormalInfo[i]= (float)(0.5-0.0291);  
		}
		else if(i<480)
		{
			NormalInfo[i]= (float)(0.5-0.0078);  
		}
		else if(i<520)
		{
			NormalInfo[i]= (float)(0.5-0.0015);
		}
		else 
		{
			NormalInfo[i]=0;
		}
	}
	Lines  =wLineCount;
    ImgAddr=lpLine;
	
	if(ScanMode==1)
	{//for Gray mode
		unsigned char *ptColor=(unsigned char*)malloc(LineBytes*3);
		unsigned char *ptColor1=ptColor;				//First  Buffer Line of original data
        unsigned char *ptColor2=&ptColor[LineBytes];	//Second Buffer Line of original data
		unsigned char *ptColor3=&ptColor[LineBytes*2];	//Third  Buffer Line of original data 
		
		//Sharpen the first line///////////////////////////////////////
		//Copy the first ,second and third line data to buffer

		memcpy(ptColor2, ImgAddr, LineBytes);
		memcpy(ptColor3, ImgAddr + LineBytes, LineBytes);

/*		for(Col=0;Col< LineBytes;Col++)
		{
			*(ptColor2+Col)=(BYTE)(*( ImgAddr+Col));				
			*(ptColor3+Col)=(BYTE)(*( ImgAddr+LineBytes+Col));			
		}//end for-loop of Copy the image data to buffer
*/		
		//deal with first and last point of first line
		//1.first point 
		TmpColor1=( (*(ptColor2)- *(ptColor2+1))+(*(ptColor2)- *(ptColor3)));
		TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
		TmpColor2=(BYTE)(*(ImgAddr));
		if( (TmpColor2+TmpColor1)<=0)
		{
			*(ImgAddr)=0;
		}
		else if((TmpColor2+TmpColor1) >=255)
		{
			*(ImgAddr)=255;
		}
		else
		{
			*(ImgAddr)+=TmpColor1;
		}
		
		//2.last point
		TmpColor1= ((*(ptColor2+LineBytes-1) - *(ptColor2+LineBytes-2))+
			         (*(ptColor2+LineBytes-1) - *(ptColor3+LineBytes-1)) ) ;
		TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
		TmpColor2=(BYTE)(*( ImgAddr+LineBytes-1));
		if( ( TmpColor2+TmpColor1 )<=0  )
		{
			*(ImgAddr+LineBytes-1)=0;
		}
		else if( (TmpColor2+TmpColor1) >=255 )
		{
			*(ImgAddr+LineBytes-1)=255;
		}
		else
		{
			*(ImgAddr+LineBytes-1)+=TmpColor1;
		}
		
		//3.other point 
		for(Col=1;Col< (LineBytes-1) ;Col++)
		{
			TmpColor1= ((*(ptColor2+Col)-*(ptColor2+Col-1))+
						(*(ptColor2+Col)-*(ptColor2+Col+1))+			        
						(*(ptColor2+Col)-*(ptColor3+Col  )));
			TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
			TmpColor2=(BYTE)(*( ImgAddr+Col));
			if( ( TmpColor2+TmpColor1 )<=0  )
			{
				*( ImgAddr+Col)=0;
			}
			else if( (TmpColor2+TmpColor1) >=255 )
			{
				*( ImgAddr+Col)=255;
			}
			else
			{
				*( ImgAddr+Col)+=TmpColor1;
			}
        }
		//end Sharpen the first Line data//////////////////////////////
		
		//sharpen the data from second lne to the last second line////
		for(Row = 1; Row < Lines - 1; Row++)
		{
			LPBYTE lpTemp = ptColor1;
			ptColor1 = ptColor2;
			ptColor2 = ptColor3;
			ptColor3 = lpTemp;
			
			//Copy the Image data to buffer
			memcpy(ptColor3, ImgAddr + LineBytes * (Row + 1), LineBytes);
/*			for(Col=0;Col< LineBytes;Col++)
			{
				*(ptColor3+Col)=(BYTE)(*(ImgAddr+LineBytes*(Row+1)+Col));
			}//end for-loop of Copy the image data to buffer
*/			
			//deal with the first point///////////////////////////////
			TmpColor1=( (*(ptColor2)- *(ptColor2+1))+
						(*(ptColor2)- *(ptColor1))  +
						(*(ptColor2)- *(ptColor3))    );
			
			TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
			
			TmpColor2=(BYTE)(*(ImgAddr+Row*LineBytes));
			if( (TmpColor2+TmpColor1)<=0)
			{
				*(ImgAddr+Row*LineBytes)=0;
			}
			else if((TmpColor2+TmpColor1) >=255 )
			{
				*(ImgAddr+Row*LineBytes)=255;
			}
			else
			{
				*(ImgAddr+Row*LineBytes)+=TmpColor1;
			}
			
			//deal with the last point///////////////////////////////
			TmpColor1=( (*(ptColor2+LineBytes-1)- *(ptColor2+LineBytes-2))+
						(*(ptColor2+LineBytes-1)- *(ptColor1+LineBytes-1))  +
						(*(ptColor2+LineBytes-1)- *(ptColor3+LineBytes-1))    );
			
			TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
			
			TmpColor2=(BYTE)(*(ImgAddr+(Row+1)*LineBytes-1));
			if( (TmpColor2+TmpColor1)<=0)
			{
				*(ImgAddr+(Row+1)*LineBytes-1)=0;
			}
			else if((TmpColor2+TmpColor1) >=255 )
			{
				*(ImgAddr+(Row+1)*LineBytes-1)=255;
			}
			else
			{
				*(ImgAddr+(Row+1)*LineBytes-1)+=TmpColor1;
			}
			
			//deal with the other point
			for(Col=1;Col<(LineBytes-1);Col++)
			{
				TmpColor1=( (*(ptColor2+Col)- *(ptColor2+Col-1))+
							(*(ptColor2+Col)- *(ptColor2+Col+1))+
							(*(ptColor2+Col)- *(ptColor1+Col))  +
							(*(ptColor2+Col)- *(ptColor3+Col))    ) ;
				
				TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
				TmpColor2=(BYTE)(*(ImgAddr+Row*LineBytes+Col));
				if( (TmpColor2+TmpColor1)<=0)
				{
					*(ImgAddr+Row*LineBytes+Col)=0;
				}
				else if((TmpColor2+TmpColor1) >=255 )
				{
					*(ImgAddr+Row*LineBytes+Col)=255;
				}
				else
				{
					*(ImgAddr+Row*LineBytes+Col)+=TmpColor1;
				}				
			}//end for-loop of other point
			
		}//end sharpen the data from second lne to the last second line
		
		//Sharpen the last line data /////////////////////////////////////
		//deal with first and last point of last line
		//1.first point 
		TmpColor1=( (*(ptColor3)- *(ptColor3+1))+
			(*(ptColor3)- *(ptColor2  )) );
		TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
		TmpColor2=(BYTE)(*(ImgAddr+(Lines-1)*LineBytes));
		if( (TmpColor2+TmpColor1)<=0)
		{
			*(ImgAddr+(Lines-1)*LineBytes)=0;
		}
		else if((TmpColor2+TmpColor1) >=255 )
		{
			*(ImgAddr+(Lines-1)*LineBytes)=255;
		}
		else
		{
			*(ImgAddr+(Lines-1)*LineBytes)+=TmpColor1;
		}
		
		//2.last point
		TmpColor1= ((*(ptColor3+LineBytes-1) - *(ptColor3+LineBytes-2))+
			(*(ptColor3+LineBytes-1) - *(ptColor2+LineBytes-1))  ) ;
		TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
		
		TmpColor2=(BYTE)(*( ImgAddr+(Lines*LineBytes)-1));
		if( ( TmpColor2+TmpColor1 )<=0  )
		{
			*(ImgAddr+(Lines*LineBytes)-1)=0;
		}
		else if( (TmpColor2+TmpColor1) >=255 )
		{
			*(ImgAddr+(Lines*LineBytes)-1)=255;
		}
		else
		{
			*(ImgAddr+(Lines*LineBytes)-1)+=TmpColor1;
		}
		
		//3.other point 
		for(Col=1;Col< (LineBytes-1) ;Col++)
		{
			TmpColor1= (  (*(ptColor3+Col)-*(ptColor3+Col-1))+
						(*(ptColor3+Col)-*(ptColor3+Col+1))+			        
						(*(ptColor3+Col)-*(ptColor2+Col))    ) ;
			TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
			
			TmpColor2=(BYTE)(*(ImgAddr+(Lines-1)*LineBytes+Col));
			if( ( TmpColor2+TmpColor1 )<=0  )
			{
				*(ImgAddr+(Lines-1)*LineBytes+Col)=0;
			}
			else if( (TmpColor2+TmpColor1) >=255 )
			{
				*(ImgAddr+(Lines-1)*LineBytes+Col)=255;
			}
			else
			{
				*(ImgAddr+(Lines-1)*LineBytes+Col)+=TmpColor1;
			}
        }		
		//end of the last line data sharpness ////////////////////////////
		
		free(ptColor);
	}
	else if(ScanMode==2)
	{  //for color mode with 24 bits image data
		
		unsigned char *ptColor=(unsigned char*)malloc(LineBytes*3);
		unsigned char *ptColor1=ptColor;				//First  Buffer Line of original data
        unsigned char *ptColor2=&ptColor[LineBytes];	//Second Buffer Line of original data
		unsigned char *ptColor3=&ptColor[LineBytes*2];	//Third  Buffer Line of original data 
		
		//Sharpen the first line///////////////////////////////////////
		//Copy the first ,second line data to buffer ptColor2 & ptColor3
		
		memcpy(ptColor2, ImgAddr, LineBytes);
		memcpy(ptColor3, ImgAddr + LineBytes, LineBytes);

/*		for(Col=0;Col< LineBytes;Col++)
		{
			*(ptColor2+Col)=(BYTE)(*( ImgAddr+Col));				
			*(ptColor3+Col)=(BYTE)(*( ImgAddr+LineBytes+Col));			
		}//end for-loop of Copy the image data to buffer
*/		
		//deal with first and last point of first line
		for(Index=0;Index<3;Index++)
		{
			//1.first point 
			TmpColor1=(( (*(ptColor2+Index)- *(ptColor2+3+Index))+
							(*(ptColor2+Index)- *(ptColor3+Index  ))   )) ;
			TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
			
			TmpColor2=(BYTE)(*(ImgAddr+Index));
			if( (TmpColor2+TmpColor1)<=0)
			{
				*(ImgAddr+Index)=0;
			}
			else if((TmpColor2+TmpColor1) >=255 )

			{
				*(ImgAddr+Index)=255;
			}
			else
			{
				*(ImgAddr+Index)+=TmpColor1;
			}
			//2.last point
			TmpColor1=( ((*(ptColor2+(LineBytes-3)+Index ) - *(ptColor2+(LineBytes-6)+Index))+
						(*(ptColor2+(LineBytes-3)+Index ) - *(ptColor3+(LineBytes-3)+Index))  ) ) ;
			TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
			
			TmpColor2=(BYTE)(*( ImgAddr+LineBytes-3+Index));
			if( ( TmpColor2+TmpColor1 )<=0  )
			{
				*(ImgAddr+LineBytes-3+Index)=0;
			}
			else if( (TmpColor2+TmpColor1) >=255 )
			{
				*(ImgAddr+LineBytes-3+Index)=255;
			}
			else
			{
				*(ImgAddr+LineBytes-3+Index)+=TmpColor1;
			}
		}//end for-loop of Index
		
		//3.other point 
		for(Col=3;Col< (LineBytes-1) ;Col+=3)
		{
			for(Index=0;Index<3;Index++)
			{
				TmpColor1=( (  (*(ptColor2+Col+Index)-*(ptColor2+Col-3+Index))+
							(*(ptColor2+Col+Index)-*(ptColor2+Col+3+Index))+			        
							(*(ptColor2+Col+Index)-*(ptColor3+Col+Index  ))  ) ) ;
				TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
				
				TmpColor2=(BYTE)(*( ImgAddr+Col+Index));
				if( ( TmpColor2+TmpColor1 )<=0  )
				{
					*( ImgAddr+Col+Index)=0;
				}
				else if( (TmpColor2+TmpColor1) >=255 )
				{
					*( ImgAddr+Col+Index)=255;
				}
				else
				{
					*( ImgAddr+Col+Index)+=TmpColor1;
				}
			}//end of for-loop for Index		
        }
		//end Sharpen the first Line data//////////////////////////////
		
		//sharpen the data from second lne to the last second line////
		for(Row=1;Row< (Lines-1);Row++)

		{
			LPBYTE lpTemp = ptColor1;
			ptColor1 = ptColor2;
			ptColor2 = ptColor3;
			ptColor3 = lpTemp;
			
			//Copy the Image data to buffer

			memcpy(ptColor3, ImgAddr + LineBytes * (Row + 1), LineBytes);

/*			for(Col=0;Col< LineBytes;Col++)
			{
				*(ptColor3+Col)=(BYTE)(*(ImgAddr+LineBytes*(Row+1)+Col));
			}//end for-loop of Copy the image data to buffer
*/			
			//deal with the first and last point///////////////////////////////
			for(Index=0;Index<3;Index++)
			{
				TmpColor1=(( (*(ptColor2+Index)- *(ptColor2+3+Index))+
							(*(ptColor2+Index)- *(ptColor1+Index  ))+
							(*(ptColor2+Index)- *(ptColor3+Index  ))  )) ;
				TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
				
				TmpColor2=(BYTE)(*(ImgAddr+Row*LineBytes+Index));
				if( (TmpColor2+TmpColor1)<=0)
				{
					*(ImgAddr+Row*LineBytes+Index)=0;
				}
				else if((TmpColor2+TmpColor1) >=255 )
				{
					*(ImgAddr+Row*LineBytes+Index)=255;
				}
				else
				{
					*(ImgAddr+Row*LineBytes+Index)+=TmpColor1;
				}
				
				//deal with the last point///////////////////////////////
				TmpColor1=(( (*(ptColor2+LineBytes-3+Index)- *(ptColor2+LineBytes-6+Index))+
							(*(ptColor2+LineBytes-3+Index)- *(ptColor1+LineBytes-3+Index))+
							(*(ptColor2+LineBytes-3+Index)- *(ptColor3+LineBytes-3+Index))    )) ;
				TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
				
				TmpColor2=(BYTE)(*(ImgAddr+(Row+1)*LineBytes-3+Index));
				if( (TmpColor2+TmpColor1)<=0)
				{
					*(ImgAddr+(Row+1)*LineBytes-3+Index)=0;
				}
				else if((TmpColor2+TmpColor1) >=255 )
				{
					*(ImgAddr+(Row+1)*LineBytes-3+Index)=255;
				}
				else
				{
					*(ImgAddr+(Row+1)*LineBytes-3+Index)+=TmpColor1;
				}
			}//end for Index for-loop
			
			//deal with the other point
			for(Col=3;Col<(LineBytes-1);Col+=3)
			{				
				for(Index=0;Index<3;Index++)
				{
					TmpColor1=(( (*(ptColor2+Col+Index)- *(ptColor2+Col-3+Index))+
								(*(ptColor2+Col+Index)- *(ptColor2+Col+3+Index))+
								(*(ptColor2+Col+Index)- *(ptColor1+Col+Index))  +
								(*(ptColor2+Col+Index)- *(ptColor3+Col+Index)) )) ;
					TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
					
					TmpColor2=(BYTE)(*(ImgAddr+Row*LineBytes+Col+Index));
					if( (TmpColor2+TmpColor1)<=0)
					{
						*(ImgAddr+Row*LineBytes+Col+Index)=0;
					}
					else if((TmpColor2+TmpColor1) >=255 )
					{
						*(ImgAddr+Row*LineBytes+Col+Index)=255;
					}
					else
					{
						*(ImgAddr+Row*LineBytes+Col+Index)+=TmpColor1;
					}
				}//end Index for-loop
				
			}//end for-loop of other point
		}//end sharpen the data from second lne to the last second line
		
		//Sharpen the last line data /////////////////////////////////////
		//deal with first and last point of last line
		for(Index=0;Index<3;Index++)
		{
			//1.first point 
			TmpColor1=(( (*(ptColor3+Index)- *(ptColor3+3+Index))+
				(*(ptColor3+Index)- *(ptColor2+Index  )) )) ;
			TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
			
			TmpColor2=(BYTE)(*(ImgAddr+(Lines-1)*LineBytes+Index));
			if( (TmpColor2+TmpColor1)<=0)
			{
				*(ImgAddr+(Lines-1)*LineBytes+Index)=0;
			}
			else if((TmpColor2+TmpColor1) >=255 )
			{
				*(ImgAddr+(Lines-1)*LineBytes+Index)=255;
			}
			else
			{
				*(ImgAddr+(Lines-1)*LineBytes+Index)+=TmpColor1;
			}
			
			//2.last point
			TmpColor1=( ((*(ptColor3+LineBytes-3+Index) - *(ptColor3+LineBytes-6+Index))+
				(*(ptColor3+LineBytes-3+Index) - *(ptColor2+LineBytes-3+Index)))) ;
			TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
			
			TmpColor2=(BYTE)(*( ImgAddr+(Lines*LineBytes)-3+Index));
			if( ( TmpColor2+TmpColor1 )<=0  )
			{
				*(ImgAddr+(Lines*LineBytes)-3+Index)=0;
			}
			else if( (TmpColor2+TmpColor1) >=255 )
			{
				*(ImgAddr+(Lines*LineBytes)-3+Index)=255;
			}
			else
			{
				*(ImgAddr+(Lines*LineBytes)-3+Index)+=TmpColor1;
			}
			
			//3.other point 
			for(Col=3;Col< (LineBytes-1) ;Col+=3)
			{
				TmpColor1=( (  (*(ptColor3+Col+Index)-*(ptColor3+Col-3+Index))+
					(*(ptColor3+Col+Index)-*(ptColor3+Col+3+Index))+			        
					(*(ptColor3+Col+Index)-*(ptColor2+Col+Index)) ) ) ;
				TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
				
				TmpColor2=(BYTE)(*(ImgAddr+(Lines-1)*LineBytes+Col+Index));
				if( ( TmpColor2+TmpColor1 )<=0  )
				{
					*(ImgAddr+(Lines-1)*LineBytes+Col+Index)=0;
				}
				else if( (TmpColor2+TmpColor1) >=255 )
				{
					*(ImgAddr+(Lines-1)*LineBytes+Col+Index)=255;
				}
				else
				{
					*(ImgAddr+(Lines-1)*LineBytes+Col+Index)+=TmpColor1;
				}
			}	
		}//end Index for-loop
		//end of the last line data sharpness ////////////////////////////
		
		free(ptColor);
	}//end for if-statement of ScanMode=2

	XDBG(DEBUG_SPICALL,"MustScanner_CrossSharpen: leave MustScanner_CrossSharpen");
}

/**********************************************************************
Author: Jack             Date: 2005/05/15
Routine Description: 
	Read the data from scanner
Parameters:
	none
Return value: 
	if operation is success
	return TRUE
	else
	return FALSE
***********************************************************************/
void* MustScanner_ReadDataFromScanner()
{
	XDBG(DEBUG_SPICALL,"MustScanner_ReadDataFromScanner: call in, and in new thread");
	WORD wTotalReadImageLines=0;
	WORD wWantedLines=g_Height;
	LPBYTE lpReadImage=g_lpReadImageHead;
	BOOL isWaitImageLineDiff=FALSE;
	DWORD wMaxScanLines=g_wMaxScanLines;
	WORD wReadImageLines=0;
	WORD wScanLinesThisBlock;
	WORD wBufferLines=g_wLineDistance*2+g_wPixelDistance;

	while(wTotalReadImageLines<wWantedLines&&g_lpReadImageHead)
	{			
		if(!isWaitImageLineDiff)
		{
			wScanLinesThisBlock=(wWantedLines-wTotalReadImageLines) < g_wScanLinesPerBlock ?
								(wWantedLines-wTotalReadImageLines) : g_wScanLinesPerBlock ;
		
			XDBG(DEBUG_SPICALL,"MustScanner_ReadDataFromScanner: wWantedLines=%d",wWantedLines);
		
			XDBG(DEBUG_SPICALL,"MustScanner_ReadDataFromScanner: wScanLinesThisBlock=%d",wScanLinesThisBlock);
	
			if(STATUS_GOOD!=Asic_ReadImage(&g_chip,lpReadImage,wScanLinesThisBlock))
			{
				XDBG(DEBUG_SPICALL,"MustScanner_ReadDataFromScanner:Asic_ReadImage return error");
				XDBG(DEBUG_SPICALL,"MustScanner_ReadDataFromScanner:thread exit");
				return NULL;
			}

			//has read in memroy Buffer
			wReadImageLines+=wScanLinesThisBlock;   
			
//			g_dwScannedTotalLines+=wScanLinesThisBlock;  
			AddScannedLines(wScanLinesThisBlock);
			
			wTotalReadImageLines+=wScanLinesThisBlock; 
			
			lpReadImage+=wScanLinesThisBlock*g_BytesPerRow;	

			//Buffer is full
			//if(wReadImageLines+g_wScanLinesPerBlock>=wMaxScanLines) //jason add g_wScanLinesPerBlock
			if(wReadImageLines>=wMaxScanLines) 
			{
				lpReadImage=g_lpReadImageHead;
				wReadImageLines=0;
			}

			//if speed of do-line-distance is lower than speed of scan
/*			
			if((WORD)(g_dwScannedTotalLines - g_wtheReadyLines) 
				>= (wMaxScanLines - (wBufferLines + g_wScanLinesPerBlock))
				&& g_dwScannedTotalLines > (DWORD)(g_wtheReadyLines) )
*/
			if((g_dwScannedTotalLines - GetReadyLines()) 
				>= (wMaxScanLines - (wBufferLines + g_wScanLinesPerBlock))
				&& g_dwScannedTotalLines > GetReadyLines())
			{
				isWaitImageLineDiff = TRUE;  
			}
		}
		else if(g_dwScannedTotalLines <= GetReadyLines()+wBufferLines+g_wScanLinesPerBlock)
		{
			isWaitImageLineDiff = FALSE;   
		}

		//test whether other thread want to cancel this thread
		pthread_testcancel();  
	}

	XDBG(DEBUG_SPICALL,"MustScanner_ReadDataFromScanner: Read image ok");
	XDBG(DEBUG_SPICALL,"MustScanner_ReadDataFromScanner: thread exit");
	XDBG(DEBUG_SPICALL,"MustScanner_ReadDataFromScanner: leave MustScanner_ReadDataFromScanner");
	return NULL;
}

/**********************************************************************
Author: Jack            Date: 2005/05/26
Routine Description: 
	get the lines of scanned
Parameters:
	none
Return value: 
	the lines of scanned
***********************************************************************/
DWORD GetScannedLines()
{
	DWORD dwScannedLines = 0;

	pthread_mutex_lock(&g_scannedLinesMutex);
	dwScannedLines = g_dwScannedTotalLines;
	pthread_mutex_unlock(&g_scannedLinesMutex);

	return dwScannedLines;
}

/**********************************************************************
Author: Jack            Date: 2005/05/26
Routine Description: 
	get lines which pass to superstratum
Parameters:
	none
Return value:
	the lines which pass to superstratum
***********************************************************************/
DWORD GetReadyLines()
{
	DWORD dwReadyLines = 0;

	pthread_mutex_lock(&g_readyLinesMutex);
	dwReadyLines = g_wtheReadyLines;
	pthread_mutex_unlock(&g_readyLinesMutex);

	return dwReadyLines;
}

/**********************************************************************
Author: Jack            Date: 2005/05/26
Routine Description: 
	add the scanned total lines
Parameters:
	wAddLines: add the lines
Return value: 
	none
***********************************************************************/
void  AddScannedLines(WORD wAddLines)
{
	pthread_mutex_lock(&g_scannedLinesMutex);

	g_dwScannedTotalLines += wAddLines;

	pthread_mutex_unlock(&g_scannedLinesMutex);	
}

/**********************************************************************
Author: Jack            Date: 2005/05/26
Routine Description: 
	add the ready lines
Parameters:
	none
Return value: 
	none
***********************************************************************/
void  AddReadyLines()
{
	pthread_mutex_lock(&g_readyLinesMutex);
	g_wtheReadyLines++;
	pthread_mutex_unlock(&g_readyLinesMutex);
}

/**********************************************************************
Author: Jack             Date: 2005/05/15
Routine Description: 
	Modifiy the image data	
Parameters:
	A: the input the image data
	B: the input the iamge data
Return value:
	the modified data
***********************************************************************/
BYTE  QBET4(BYTE A, BYTE B)
{
	BYTE bQBET[16][16]= {
	0, 0, 0, 0, 1, 1, 2, 2, 4, 4, 5, 5, 8, 8, 9, 9,
	0, 0, 0, 0, 1, 1, 2, 2, 4, 4, 5, 5, 8, 8, 9, 9,
	0, 0, 0, 0, 1, 1, 2, 2, 4, 4, 5, 5, 8, 8, 9, 9,
	0, 0, 0, 0, 1, 1, 2, 2, 4, 4, 5, 5, 8, 8, 9, 9,
	1, 1, 1, 1, 3, 3, 3, 3, 6, 6, 6, 6,10,10,11,11,
	1, 1, 1, 1, 3, 3, 3, 3, 6, 6, 6, 6,10,10,11,11,
	2, 2, 2, 2, 3, 3, 3, 3, 7, 7, 7, 7,10,10,11,11,
	2, 2, 2, 2, 3, 3, 3, 3, 7, 7, 7, 7,10,10,11,11,
	4, 4, 4, 4, 6, 6, 7, 7,12,12,12,12,13,13,14,14,
	4, 4, 4, 4, 6, 6, 7, 7,12,12,12,12,13,13,14,14,
	5, 5, 5, 5, 6, 6, 7, 7,12,12,12,12,13,13,14,14,
	5, 5, 5, 5, 6, 6, 7, 7,12,12,12,12,13,13,14,14,
	8, 8, 8, 8,10,10,10,10,13,13,13,13,15,15,15,15,
	8, 8, 8, 8,10,10,10,10,13,13,13,13,15,15,15,15,
	9, 9, 9, 9,11,11,11,11,14,14,14,14,15,15,15,15,
	9, 9, 9, 9,11,11,11,11,14,14,14,14,15,15,15,15};

	A=A&0x0f;
	B=B&0x0f;
	return bQBET[A][B];
} /* end of the file MustScanner.c */
