/**********************************************************************
Copyright (C) 2005 Mustek Corporation, All rights reserved
    
Filename: spicall.c   [Abstract: ]

Author: Jack            Date: 2005/05/13
	
Function Description:
	Program operate the device

[Operative Environment:]

[Reference Articles:]

Revision History:
*********************************************************************/
/* sane - Scanner Access Now Easy.

   Copyright (C) 2005 Mustek.
   Originally maintained by Mustek
   Author:Roy 2005.5.24

   This file is part of the SANE package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   As a special exception, the authors of SANE give permission for
   additional uses of the libraries contained in this release of SANE.

   The exception is that, if you link a SANE library with other files
   to produce an executable, this does not by itself cause the
   resulting executable to be covered by the GNU General Public
   License.  Your use of that executable is in no way restricted on
   account of linking the SANE library code into it.

   This exception does not, however, invalidate any other reasons why
   the executable file might be covered by the GNU General Public
   License.

   If you submit changes to SANE to the maintainers to be included in
   a subsequent release, you agree by submitting the changes that
   those changes may be distributed with this exception intact.

   If you write modifications of your own for SANE, it is your choice
   whether to permit this exception to apply to your modifications.
   If you do not wish that, delete this exception notice.

*/
#include <stdlib.h>
#include <math.h>

#include "../include/spicall.h"
#include "../include/typedef.h"
#include "../include/ini.h"
#include "../include/asic.h"
#include "../include/debug.h"
#include "ScannerDefine.h"

#include "spicall_ex.h"
#include "MustScanner.h"
#include "ScannerReflective.h"
#include "ScannerTransparent.h"

void AutoLevel(LPSTR lpSource, SCANMODE scanMode, WORD ScanLines, DWORD BytesPerLine);
void QBETDetectAutoLevel(void* pDIB, DWORD ImageWidth, DWORD ImageHeight);
void QBetChange(LPSTR lpSource, SCANMODE scanMode, WORD ScanLines, DWORD BytesPerLine);

/**********************************************************************
Author: Jack            Date: 2005/05/13
Routine Description: 
	Open and initialize the device
Parameters:
	none
Return value: 
	if open the device success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL _Open()
{
	XDBG(DEBUG_SPICALL, "_Open: call in");
	g_pDeviceFile = (LPBYTE)malloc(200);
	
	if(NULL == g_pDeviceFile)
	{
		return FALSE;
	}
	
	OptionReadString("[spicall]", "DeviceFileName", "/dev/usb/scanner0", 
						g_pDeviceFile, 200, INIFILENAME);
	
	XDBG(DEBUG_SPICALL, "_Open: leave _Open");
	return MustScanner_Init();	
}

/**********************************************************************
Author: Jack            Date: 2005/05/13
Routine Description: 
	Close the device
Parameters:
	none
Return value: 
	if close the device success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL _Close()
{
	XDBG(DEBUG_SPICALL, "_Close: call in");
	if(NULL != g_pDeviceFile)
	{
		free(g_pDeviceFile);
		g_pDeviceFile = NULL;
	}
	XDBG(DEBUG_SPICALL, "_Close: leave _Close");
	
	return TRUE;
}

/**********************************************************************
Author: Jack            Date: 2005/05/13
Routine Description: 
	Check the device connect status
Parameters:
	none
Return value: 
	if the device is connected
	return TRUE
	else 
	return FALSE
***********************************************************************/
BOOL GetDeviceStatus()
{
	XDBG(DEBUG_SPICALL, "GetDeviceStatus: call in");
	return MustScanner_GetScannerState();
}

/**********************************************************************
Author: Jack            Date: 2005/05/13
Routine Description: 
	Turn the lamp on or off
Parameters:
	isLampOn: turn the lamp on or off
	isTALampOn: turn the TA lamp on or off
Return value: 
	if operation success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL PowerControl(BOOL isLampOn, BOOL isTALampOn)
{
	XDBG(DEBUG_SPICALL, "PowerControl: call in");
	return MustScanner_PowerControl(isLampOn, isTALampOn);
}

/**********************************************************************
Author: Jack            Date: 2005/05/13
Routine Description: 
	Turn the carriage home
Parameters:
	none
Return value: 
	if the operation success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL CarriageHome()
{
	XDBG(DEBUG_SPICALL, "CarriageHome: call in");
	return MustScanner_BackHome();
}

/**********************************************************************
Author: Jack            Date: 2005/05/13
Routine Description: 
	Get gamma input/output bit count
Parameters:
	pGammaInfo: the gamma information
Return value: 
	if the operation success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL GetGammaInfo(LPGAMMAINFO pGammaInfo)
{
	XDBG(DEBUG_SPICALL,"GetGammaInfo: call in ");
	
	switch(pGammaInfo->smScanMode) 
	{
		case SM_GRAY:
			pGammaInfo->wInputGammaBits  = 12;
			pGammaInfo->wOutputGammaBits = 8;
			break;
		case SM_RGB24:
			pGammaInfo->wInputGammaBits  = 12;
			pGammaInfo->wOutputGammaBits = 8;
			break;
		case SM_GRAY16:
			pGammaInfo->wInputGammaBits  = 16;
			pGammaInfo->wOutputGammaBits = 16;
			break;
		case SM_RGB48:
			pGammaInfo->wInputGammaBits  = 16;
			pGammaInfo->wOutputGammaBits = 16;
			break;
		default:
			pGammaInfo->wInputGammaBits  = 0;
			pGammaInfo->wOutputGammaBits = 0;
			return FALSE;
	}
	
	XDBG(DEBUG_SPICALL, "GetGammaInfo: leave GetGammaInfo");
	return TRUE;
}

/**********************************************************************
Author: Jack            Date: 2005/05/13
Routine Description: 
	set scan parameters
Parameters:
	pSetParameters: the information of scaning
Return value: 
	if the operation success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL SetParameters (LPSETPARAMETERS pSetParameters)
{
	XDBG(DEBUG_SPICALL,"SetParameters: call in");

	WORD X1inTargetDpi;
	WORD Y1inTargetDpi;
	WORD X2inTargetDpi;
	WORD Y2inTargetDpi;
	
	//0. Reset
	if(ST_Reflective == g_ScanType)
	{
		Reflective_Reset();			 
	}
	else
	{
		Transparent_Reset();			
	}
	
	//1. Scan mode
	switch(pSetParameters->smScanMode) 
	{


		case SM_TEXT:
			g_tiTarget.cmColorMode = CM_TEXT;
			break;
		case SM_GRAY:
			g_tiTarget.cmColorMode = CM_GRAY8;
			break;
		case SM_GRAY16:
			g_tiTarget.cmColorMode = CM_GRAY16;
			break;
		case SM_RGB24:
			g_tiTarget.cmColorMode = CM_RGB24;
			break;
		case SM_RGB48:
			g_tiTarget.cmColorMode = CM_RGB48;
			break;
		default:
			return FALSE;
	}
	
	//2. Scan source
	g_ssScanSource = pSetParameters->ssScanSource;
	g_tiTarget.bScanSource = pSetParameters->ssScanSource;
	
	if (SS_Reflective == pSetParameters->ssScanSource)
	{
		g_ScanType = ST_Reflective;
	}
	else if(SS_Positive == pSetParameters->ssScanSource
			|| SS_Negative == pSetParameters->ssScanSource
			|| SS_ADF == pSetParameters->ssScanSource)
	{
		g_ScanType = ST_Transparent;
	}
	else
	{
		XDBG(DEBUG_SPICALL,"SetParameters: ScanSource error");
		return FALSE;
	}
	
	//3. pixel flavor
	if (PF_BlackIs0 == pSetParameters->pfPixelFlavor
		|| PF_WhiteIs0 == pSetParameters->pfPixelFlavor) 
	{	
		g_PixelFlavor = pSetParameters->pfPixelFlavor;		
	}
	else
	{
		XDBG(DEBUG_SPICALL,"SetParameters: PixelFlavor error");
		return FALSE;
	}
	
	//4. Scan area
	if(pSetParameters->fmArea.x1 >= pSetParameters->fmArea.x2) 
	{	
		XDBG(DEBUG_SPICALL,"SetParameters: x1 > x2, error");
		return FALSE;
	}
	if(pSetParameters->fmArea.y1 >= pSetParameters->fmArea.y2) 
	{
		XDBG(DEBUG_SPICALL,"SetParameters: y1 >= y2, error");
		return FALSE;
	}
	if(pSetParameters->fmArea.x2 > MAX_SCANNING_WIDTH) //Just for A4 size
	{
		XDBG(DEBUG_SPICALL,"SetParameters: x2 > MAX_SCANNING_WIDTH, error");
		return FALSE;
	}
	if (pSetParameters->fmArea.y2 > MAX_SCANNING_HEIGHT)//Just for A4 size  
	{	
		XDBG(DEBUG_SPICALL,"SetParameters: y2 > MAX_SCANNING_HEIGHT, error");
		return FALSE;
	}
	
	X1inTargetDpi=(WORD)((DWORD)(pSetParameters->fmArea.x1)*(DWORD)(pSetParameters->wTargetDPI)/300L);
	Y1inTargetDpi=(WORD)((DWORD)(pSetParameters->fmArea.y1)*(DWORD)(pSetParameters->wTargetDPI)/300L);
	X2inTargetDpi=(WORD)((DWORD)(pSetParameters->fmArea.x2)*(DWORD)(pSetParameters->wTargetDPI)/300L);
	Y2inTargetDpi=(WORD)((DWORD)(pSetParameters->fmArea.y2)*(DWORD)(pSetParameters->wTargetDPI)/300L);

	g_tiTarget.isOptimalSpeed = TRUE;
	g_tiTarget.wDpi		= pSetParameters->wTargetDPI;
	g_tiTarget.wX		= X1inTargetDpi;
	g_tiTarget.wY		= Y1inTargetDpi;
	g_tiTarget.wWidth	= X2inTargetDpi - X1inTargetDpi;           
	g_tiTarget.wHeight	= Y2inTargetDpi - Y1inTargetDpi;
	
	XDBG(DEBUG_SPICALL,"SetParameters: g_tiTarget.wDpi=%d",    g_tiTarget.wDpi);
	XDBG(DEBUG_SPICALL,"SetParameters: g_tiTarget.wX=%d",      g_tiTarget.wX);
	XDBG(DEBUG_SPICALL,"SetParameters: g_tiTarget.wY=%d",      g_tiTarget.wY);
	XDBG(DEBUG_SPICALL,"SetParameters: g_tiTarget.wWidth=%d",  g_tiTarget.wWidth);
	XDBG(DEBUG_SPICALL,"SetParameters: g_tiTarget.wHeight=%d", g_tiTarget.wHeight);
	
	//5.Prepare
	if(FALSE == MustScanner_Prepare(g_tiTarget.bScanSource))
	{	
		XDBG(DEBUG_SPICALL,"SetParameters: MustScanner_Prepare fail");
		return FALSE;
	}	
	
	//6. Linear threshold
	if(pSetParameters->wLinearThreshold>256 && pSetParameters->smScanMode==SM_TEXT)
	{
		XDBG(DEBUG_SPICALL,"SetParameters: LinearThreshold error");
		return FALSE;
	}
	else
	{
		g_wLineartThreshold = pSetParameters->wLinearThreshold;
	}
	
	//7. Gamma table
	if(NULL != pSetParameters->pGammaTable)
	{
		XDBG(DEBUG_SPICALL,"SetParameters: IN gamma table not NULL");
		g_pGammaTable = pSetParameters->pGammaTable;
		g_isSelfGamma = FALSE;
	}
	else if(pSetParameters->smScanMode==SM_GRAY || pSetParameters->smScanMode==SM_RGB24)
	{
		WORD i;
		BYTE byGammaData;
		double pow_d;
		double pow_z=(double)10/16.0;
		
		g_pGammaTable = (WORD*)malloc(sizeof(WORD)*4096*3);
		
		XDBG(DEBUG_SPICALL,"SetParameters: gamma table malloc %d Bytes ",sizeof(WORD)*4096*3);
		XDBG(DEBUG_SPICALL,"SetParameters: address of g_pGammaTable=%d",g_pGammaTable);
		
		if(NULL == g_pGammaTable)
		{
			XDBG(DEBUG_SPICALL,"SetParameters: gamma table malloc fail");
			return FALSE;
		}
		g_isSelfGamma = TRUE;
		
		for(i = 0; i < 4096; i++)
		{
			pow_d=(double)i/(double)4096;
			
			byGammaData=(BYTE)(pow(pow_d,pow_z)*255);
			
			*(g_pGammaTable+i) = byGammaData;
			*(g_pGammaTable+i+4096) = byGammaData;
			*(g_pGammaTable+i+8192) = byGammaData;
		}
	}
	else if(pSetParameters->smScanMode==SM_GRAY16 || pSetParameters->smScanMode==SM_RGB48)
	{
		unsigned int i,wGammaData;
		g_pGammaTable = (WORD*)malloc(sizeof(WORD)*65536*3);	
		
		if(g_pGammaTable==NULL)
		{
			XDBG(DEBUG_SPICALL,"SetParameters: gamma table malloc fail");
			return FALSE;
		}
		g_isSelfGamma = TRUE;
		
		for(i=0;i<65536;i++)
		{
			wGammaData=(WORD)(pow((((float)i)/65536.0),(((float)10)/16.0))*65535);
			
			*(g_pGammaTable+i) = wGammaData;
			*(g_pGammaTable+i+65536)  = wGammaData;
			*(g_pGammaTable+i+65536*2) = wGammaData;
		}
	}
	else
	{	
		XDBG(DEBUG_SPICALL,"SetParameters: set g_pGammaTable to NULL");
		g_pGammaTable = NULL;
	}
	
	//8. Sharpen
	g_bSharpen = OptionReadInt("[spicall]","SharpenEnable",0,INIFILENAME);
	
	XDBG(DEBUG_SPICALL,"SetParameters: leave SetParameters");
	return TRUE;
}

/**********************************************************************
Author: Jack            Date: 2005/05/13
Routine Description: 
	get the optical dpi and scan area
Parameters:
	pGetParameters: the information of scan
Return value: 
	if the operation is success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL GetParameters(LPGETPARAMETERS pGetParameters)
{
	XDBG(DEBUG_SPICALL,"GetParameters: call in ");
	if(ST_Reflective==g_ScanType)
	{
		if(FALSE==Reflective_ScanSuggest(&g_tiTarget,&g_ssSuggest))
		{
			XDBG(DEBUG_SPICALL,"GetParameters: Reflective_ScanSuggest error");
			return FALSE;			 
		}
	}
	else
	{
		if(FALSE==Transparent_ScanSuggest(&g_tiTarget,&g_ssSuggest))
		{
			XDBG(DEBUG_SPICALL,"GetParameters: Transparent_ScanSuggest error");
			return FALSE;			
		}
	}
	
	pGetParameters->wSourceXDPI=g_ssSuggest.wXDpi;
	pGetParameters->wSourceYDPI=g_ssSuggest.wYDpi;
	pGetParameters->dwLength=(DWORD) g_ssSuggest.wHeight;
	pGetParameters->dwLineByteWidth=g_ssSuggest.dwBytesPerRow;
	
	XDBG(DEBUG_SPICALL,"GetParameters: leave GetParameters");
	
	return TRUE;
}

/**********************************************************************
Author: Jack            Date: 2005/05/13
Routine Description: 
	start scan image
Parameters:
	none
Return value:
	if operation is success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL StartScan()
{
	XDBG(DEBUG_SPICALL,"StartScan: call in ");
	if(ST_Reflective==g_ScanType)
	{
		XDBG(DEBUG_SPICALL,"StartScan: g_ScanType==ST_Reflective ");
		
		return Reflective_SetupScan(g_ssSuggest.cmScanMode,
									g_ssSuggest.wXDpi,
									g_ssSuggest.wYDpi,
									PF_BlackIs0,
									g_ssSuggest.wX,
									g_ssSuggest.wY,
									g_ssSuggest.wWidth,
									g_ssSuggest.wHeight);
	}
	else
	{
		XDBG(DEBUG_SPICALL,"StartScan: g_ScanType==ST_Transparent ");
		
		return Transparent_SetupScan(g_ssSuggest.cmScanMode,
										g_ssSuggest.wXDpi,
										g_ssSuggest.wYDpi,
										PF_BlackIs0,
										g_ssSuggest.wX,
										g_ssSuggest.wY,
										g_ssSuggest.wWidth,
										g_ssSuggest.wHeight);
	}
}

/**********************************************************************
Author: Jack             Date: 2005/05/14
Routine Description: 
	Read the scanner data
Parameters:
	pImageRows: the information of the data
Return value:	
	if the operation is seccuss
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL ReadScannedData(LPIMAGEROWS pImageRows)
{
	XDBG(DEBUG_SPICALL,"ReadScannedData: call in ");
	BOOL isRGBInvert;
  WORD Rows = 0;
	BYTE* lpBlock = (BYTE*)pImageRows->pBuffer;
  BYTE* lpTemp = (BYTE*)pImageRows->pBuffer;
	int i;
	
	if( pImageRows->roRgbOrder == RO_RGB )
		isRGBInvert = FALSE;
	else
		isRGBInvert = TRUE;
	
	Rows = pImageRows->wWantedLineNum;	
	XDBG(DEBUG_SPICALL,"ReadScannedData: wanted Rows = %d",Rows );
	
	if(ST_Reflective==g_ScanType)
	{
		if(FALSE==Reflective_GetRows(lpBlock,&Rows,isRGBInvert))
			return FALSE;			 
	}
	else
	{
		if(FALSE==Transparent_GetRows(lpBlock,&Rows,isRGBInvert))
			return FALSE;			
	}
	
	pImageRows->wXferedLineNum = Rows;
	
	if (g_PixelFlavor==PF_WhiteIs0 || g_ScanMode==CM_TEXT || g_ssScanSource==SS_Negative) 
	{
		int TotalSize= Rows*g_ssSuggest.dwBytesPerRow;
		for(i=0;i<TotalSize;i++) 
		{
			*(lpBlock++)^=0xff;
		}
	}

	if(SS_Negative == g_ssScanSource)
	{
    XDBG(DEBUG_SPICALL,"ReadScannedData: before autolevel");
    XDBG(DEBUG_SPICALL,"ReadScannedData: Rows = %d, g_ssSuggest.dwBytesPerRow = %d", Rows, g_ssSuggest.dwBytesPerRow);
    
	  AutoLevel(lpTemp, g_ScanMode, Rows, g_ssSuggest.dwBytesPerRow);
//  QBetChange(lpTemp, g_ScanMode, Rows, g_ssSuggest.dwBytesPerRow);

    XDBG(DEBUG_SPICALL,"ReadScannedData: autolevel is ok");
	}
	
	XDBG(DEBUG_SPICALL,"ReadScannedData: leave ReadScannedData");
	
	return TRUE;
}

/**********************************************************************
Author: Jack             Date: 2005/05/14
Routine Description: 
	Stop scan
Parameters:
	none
Return value: 
	if operation is success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL StopScan()
{
	XDBG(DEBUG_SPICALL,"StopScan: call in ");
	BOOL rt;
	int i;
	
	//stop read data and kill thread
	if(ST_Reflective==g_ScanType)
	{
		rt = Reflective_StopScan();			 
	}
	else
	{
		rt = Transparent_StopScan();			 
	}
	
	//free gamma table
	if(g_isSelfGamma && g_pGammaTable!=NULL)
	{
		for(i=0;i<20;i++)
		{
			if(!g_isScanning)
			{
				free(g_pGammaTable);
				g_pGammaTable=NULL;				
				break;
			}
			else
			{
				sleep(1);  //waiting ReadScannedData return.
			}
		}
	}
	
	//free image buffer
	if(g_lpReadImageHead != NULL) 
	{
		free(g_lpReadImageHead); 
		g_lpReadImageHead = NULL;
	}
	
	XDBG(DEBUG_SPICALL,"StopScan: leave StopScan ");
	return rt;
}

/**********************************************************************
Author: Jack             Date: 2005/05/14
Routine Description: 
	Check the status of TA
Parameters:
	none
Return value: 
	if operation is success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL IsTAConnected()
{	
	XDBG(DEBUG_SPICALL,"StopScan: call in ");

	BOOL hasTA;
	if(Asic_Open(&g_chip,g_pDeviceFile)!=STATUS_GOOD)
	{
		return FALSE;
	}
	
	if(Asic_IsTAConnected(&g_chip,&hasTA)!=STATUS_GOOD)
	{
		Asic_Close(&g_chip);
		return FALSE;
	}
	
	Asic_Close(&g_chip);

	XDBG(DEBUG_SPICALL,"StopScan: leave StopScan ");
	return hasTA;
}

/**********************************************************************
Author: Jack             Date: 2005/05/14
Routine Description: 
	Get the status of the HK
Parameters:
	pKey: the status of key
Return value: 
	if the operation is success
	return TRUE
	else
	return FALSE
***********************************************************************/
BOOL GetKeyStatus(BYTE* pKey)
{
	XDBG(DEBUG_SPICALL,"GetKeyStatus: call in");

	if(STATUS_GOOD != Asic_Open(&g_chip, g_pDeviceFile))
	{
		XDBG(DEBUG_SPICALL,"GetKeyStatus: Asic_Open is fail");
		return FALSE;
	}

	BYTE pKeyTemp = 0x00;
	STATUS status = Asic_CheckFunctionKey(&g_chip, &pKeyTemp);
	if (STATUS_GOOD != status)
	{
		XDBG(DEBUG_SPICALL,"GetKeyStatus: Asic_CheckFunctionKey is fail");
		return FALSE;
	}
	
	if(0x01 == pKeyTemp)
	{
		*pKey = 0x01; //Scan key pressed
	}
	
	if(0x02 == pKeyTemp)
	{
		*pKey = 0x02; //Copy key pressed
	}
	if(0x04 == pKeyTemp)
	{
		*pKey = 0x03; //Fax key pressed
	}
	if(0x08 == pKeyTemp)
	{
		*pKey = 0x04; //Email key pressed
	}
	if(0x10 == pKeyTemp)
	{
		*pKey = 0x05; //Panel key pressed
	}

	if(STATUS_GOOD != Asic_Close(&g_chip))
	{
		XDBG(DEBUG_SPICALL,"GetKeyStatus: Asic_Close is fail");
		return FALSE;
	}
	
	XDBG(DEBUG_SPICALL,"GetKeyStatus: leave GetKeyStatus");
	return TRUE;  
}

/**********************************************************************
Author: Jack             Date: 2005/05/14
Routine Description: 
	Deal with the image with auto level	
Parameters:
	lpSource: the data of image
	scanMode: the scan mode
	ScanLines: the rows of image
	BytesPerLine: the bytes of per line
Return value: 
	none
***********************************************************************/
void AutoLevel(LPSTR lpSource, SCANMODE scanMode, WORD ScanLines, DWORD BytesPerLine)
{
  XDBG(DEBUG_SPICALL,"AutoLevel: call in");
  
	int    ii;
	DWORD  i,j;
	DWORD  tLines,CountPixels,TotalImgSize;
	WORD   R,G,B,max_R,max_G,max_B,min_R,min_G,min_B;
	float  fmax_R,fmax_G,fmax_B;
	DWORD  sum_R = 0, sum_G = 0, sum_B = 0;
	float  mean_R, mean_G, mean_B;
	DWORD  hisgram_R[256], hisgram_G[256], hisgram_B[256];
	
	DWORD iWidth=BytesPerLine/3;
	DWORD iHeight=ScanLines;
	BYTE* pbmpdata=(BYTE*)lpSource;   
	
	
	DWORD   tmp = 0;
	WORD   imin_threshold[3];
	WORD   imax_threshold[3];
	
	if (scanMode != CM_RGB24ext)
	{
		return;
	}
	
	i = j = 0;
	tLines = CountPixels = TotalImgSize = 0;
	
	TotalImgSize = iWidth * iHeight;

	//////////////////////////////////////////////////////////////////
	

	for(i=0; i<256; i++){
		
		hisgram_R[i] = 0; hisgram_G[i] = 0; hisgram_B[i] = 0;
	}

  
	//////////////////////////////////////////////////////////////////
  
  XDBG(DEBUG_SPICALL,"AutoLevel: init data is over");

	//Find min , max, mean
	max_R = max_G = max_B =0;
	min_R = min_G = min_B =255;
	tLines = 0;
  XDBG(DEBUG_SPICALL, "AutoLevel: iHeight = %d, iWidth = %d", iHeight, iWidth);
  
	for(j=0; j<iHeight; j++)
	{
		tLines = j * iWidth *3;

//    XDBG(DEBUG_SPICALL, "AutoLevel: tLines = %d", tLines);
        
		for(i=0;i<iWidth;i++)
		{
			R = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 2));
			G = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 1));

			B = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3));
			
			max_R = _MAX(R,max_R);
			max_G = _MAX(G,max_G);
			max_B = _MAX(B,max_B);
			
			min_R = _MIN(R,min_R);
			min_G = _MIN(G,min_G);
			min_B = _MIN(B,min_B);

			hisgram_R[(BYTE)R]++; hisgram_G[(BYTE)G]++; hisgram_B[(BYTE)B]++;
			
			sum_R += R; sum_G += G; sum_B += B;
			
		*(pbmpdata + (tLines + i * 3 + 2)) = (BYTE)R;
		*(pbmpdata + (tLines + i * 3 + 1)) = (BYTE)G;
	  *(pbmpdata + (tLines + i * 3))     = (BYTE)B;
			
			CountPixels++;
		}//end for      
		
	}//end for

  XDBG(DEBUG_SPICALL,"AutoLevel: Find min , max is over!");
  
	mean_R = (float)(sum_R / TotalImgSize);
	mean_G = (float)(sum_G / TotalImgSize);
	mean_B = (float)(sum_B / TotalImgSize);
	
	//////////////////////////////////////////////////////////////////
	
	imin_threshold[0] = 0;    imin_threshold[1] = 0;    imin_threshold[2] = 0;
	imax_threshold[0] = 0;    imax_threshold[1] = 0;    imax_threshold[2] = 0;
	
	for(ii = 0; ii < 256; ii++)
	{
		if(hisgram_R[ii] > 0)
			if(hisgram_R[ii] >= imin_threshold[0])
			{
				min_R = ii;
				break;
			}
			//else{
			//  tmp += hisgram_R[i];
			//}
	}
	
	tmp = 0;
	for (ii = 255; ii >= 0; ii--)
	{
		if (hisgram_R[ii] > 0)
			if(hisgram_R[ii] >= imax_threshold[0])
			{
				max_R = ii;
				break;
			}
			//else{
			//   tmp += hisgram_R[i];
			//}
	}
	
	tmp = 0;
	for (ii = 0; ii < 256; ii++)
	{
		if(hisgram_G[ii] > 0)
			if(hisgram_G[ii] >= imin_threshold[1])
			{
				min_G = ii;
				break;
			}

			//else{
			//  tmp += hisgram_G[i];
			//}
	}
	
	tmp = 0;
	for (ii = 255; ii >= 0; ii--)
	{
		if(hisgram_G[ii] > 0)
			if(hisgram_G[ii] >= imax_threshold[1])
			{
				max_G = ii;
				break;
			}
			//else{
			//   tmp += hisgram_G[i];
			//}
	}
	
	tmp = 0;
	for (ii = 0; ii < 256; ii++)
	{
		if(hisgram_B[ii] > 0)
			if(hisgram_B[ii]  >= imin_threshold[2])
			{
				min_B = ii;
				break;
			}
			//else{
			//  tmp += hisgram_B[i];
			//}
	}
	
	tmp = 0;
	for (ii = 255; ii >= 0; ii--)
	{
		if(hisgram_B[ii] > 0)
			if(hisgram_B[ii] >= imax_threshold[2])
			{
				max_B = ii;
				break;
			}
			//else{
			//  tmp += hisgram_B[i];
			//}
	}

  XDBG(DEBUG_SPICALL,"AutoLevel: Set min , max is over!");
	///////////////////////////////////////////////////////////////////////////
  
/*
  max_B = OptionReadInt("[spicall]", "MAX_R", 255, INIFILENAME);
  min_B = OptionReadInt("[spicall]", "MIN_R", 0, INIFILENAME);
  max_G = OptionReadInt("[spicall]", "MAX_G", 255, INIFILENAME);
  min_G = OptionReadInt("[spicall]", "MIN_G", 0, INIFILENAME);
  max_R = OptionReadInt("[spicall]", "MAX_B", 255, INIFILENAME);
  min_R = OptionReadInt("[spicall]", "MIN_B", 0, INIFILENAME);

  XDBG(DEBUG_SPICALL,"AutoLevel: max_R = %d, min_R = %d, max_G = %d, min_G = %d, "
        "max_B = %d, min_B = %d", max_R, min_R, max_G, min_G, max_B, min_B);
*/

  max_B = OptionReadInt("[spicall]", "MAX_R", 220, INIFILENAME);
  min_B = OptionReadInt("[spicall]", "MIN_R", 40, INIFILENAME);
  max_G = OptionReadInt("[spicall]", "MAX_G", 240, INIFILENAME);
  min_G = OptionReadInt("[spicall]", "MIN_G", 100, INIFILENAME);
  max_R = OptionReadInt("[spicall]", "MAX_B", 250, INIFILENAME);
  min_R = OptionReadInt("[spicall]", "MIN_B", 120, INIFILENAME);    

 	//Autolevel:
	sum_R = max_R-min_R;
	sum_G = max_G-min_G;
	sum_B = max_B-min_B;
	
	//////////////////////////////////////////////////////////////////////////
	
	for(j=0; j<iHeight; j++)
	{
		tLines = j * iWidth *3;
		for(i=0;i<iWidth;i++)
		{
			R = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 2));
			G = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 1));
			B = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3));
			
			//-------------------------R
			if(sum_R == 0) R = max_R;
			else if (R < min_R) R = 0;
			else if(R<=255){
				fmax_R = ((float)((R - min_R)*255)/(float)sum_R); 
				R = (WORD)fmax_R;
				fmax_R = (fmax_R - R)*10;
				
				if(fmax_R >=5)
					R++;
			}
			if (R > 255)   R = 255;
			//-------------------------G	
			if(sum_G == 0) G = max_G;
			else if (G < min_G) G = 0;
			else if(G<=255){
				fmax_G = ((float)((G - min_G)*255)/(float)sum_G); 
				G = (WORD)fmax_G;
				fmax_G = (fmax_G - G)*10;
				if(fmax_G >=5)
					G++;
				
			}
			if (G > 255)   G = 255;
			//-------------------------B
			if(sum_B == 0) B = max_B;
			else if (B < min_B) B = 0;
			else if(B<=255){
				fmax_B = ((float)(B - min_B)*255/(float)sum_B); 
				B = (WORD)fmax_B;
				fmax_B = (fmax_B - B)*10;
				if(fmax_B >= 5)
					B++;
			}
			if (B > 255)   B = 255;
			
			hisgram_R[(BYTE)R]++; hisgram_G[(BYTE)G]++; hisgram_B[(BYTE)B]++;
			
			*(pbmpdata + (tLines + i * 3 + 2)) = (BYTE)R;
			*(pbmpdata + (tLines + i * 3 + 1)) = (BYTE)G;
			*(pbmpdata + (tLines + i * 3))     = (BYTE)B;
			
		}//end for
	}//end for

    XDBG(DEBUG_SPICALL,"AutoLevel: left AutoLevel");
	return;
}

/**********************************************************************
Author: Jack             Date: 2005/05/14
Routine Description: 
	Deal with image with auto level
Parameters:
	pDIB: the data of image
	ImageWidth: the width of image
	ImageHeight: the height of image
Return value: 
	none
***********************************************************************/
void QBETDetectAutoLevel(void* pDIB, DWORD ImageWidth, DWORD ImageHeight)
{
	WORD   *pbmpdata;
	float  fRPercent = 0.0;
	float  fGPercent = 0.0;
	float  fBPercent = 0.0;
	float  fRSum,fGSum,fBSum;
	
	int    i,j;
	DWORD  tLines,CountPixels,TotalImgSize;
	WORD   R,G,B,max_R,max_G,max_B,min_R,min_G,min_B;
	WORD   wIndexR,wIndexG,wIndexB;
	float  fmax_R,fmax_G,fmax_B;
	DWORD  sum_R = 0, sum_G = 0, sum_B = 0;
	DWORD  hisgram_R[1024], hisgram_G[1024], hisgram_B[1024];
	
	if(!pDIB)
	{
		return;
	}
	
	pbmpdata    = (WORD *) pDIB;
	
	CountPixels = 0;
	TotalImgSize = ImageWidth * ImageHeight;
	
	//////////////////////////////////////////////////////////////////

	for(i=0; i<1024; i++){
		
		hisgram_R[i] = 0; hisgram_G[i] = 0; hisgram_B[i] = 0;
	}
	
	//////////////////////////////////////////////////////////////////

	//Find min , max, mean
	max_R = max_G = max_B =0;
	min_R = min_G = min_B =1023;
	tLines = 0;
	
	for(j=0; j<(int)ImageHeight; j++){
		tLines = j * ImageWidth *3;
		for(i=0;i<(int)ImageWidth;i++)
		{
			R = *(pbmpdata + (tLines + i * 3 + 2));
			G = *(pbmpdata + (tLines + i * 3 + 1));
			B = *(pbmpdata + (tLines + i * 3));
			
			max_R = _MAX(R,max_R);
			max_G = _MAX(G,max_G);
			max_B = _MAX(B,max_B);
			
			min_R = _MIN(R,min_R);
			min_G = _MIN(G,min_G);
			min_B = _MIN(B,min_B);
			
			hisgram_R[R]++; hisgram_G[G]++; hisgram_B[B]++;
			
			sum_R += R; sum_G += G; sum_B += B;
			
			*(pbmpdata + (tLines + i * 3 + 2)) = R;
			*(pbmpdata + (tLines + i * 3 + 1)) = G;
			*(pbmpdata + (tLines + i * 3))     = B;
			
			CountPixels++;
		}//end for
		
	}//end for

	///////////////////////////////////////////////////////////////////////////
	
	fRSum = 0.0;
	fGSum = 0.0;
	fBSum = 0.0;
	
	wIndexR = 511;
	wIndexG = 511;
	wIndexB = 511;
	
	for(i=0;i<1024;i++){
		fRSum += (float)hisgram_R[i];
		fRPercent = (fRSum / CountPixels)*100;
		if(fRPercent > 50) 
		{ 
			wIndexR = i;
			break;
		}
		
	}
	
	for(i=0;i<1024;i++){
		fGSum += (float)hisgram_G[i];
		fGPercent = (fGSum / CountPixels) * 100;
		if(fGPercent > 50) 
		{ 
			wIndexG = i;
			break;
		}
		
	}
	
	for(i=0;i<1024;i++){
		fBSum += (float)hisgram_B[i];
		fBPercent = (fBSum / CountPixels) * 100;
		if(fBPercent > 50) 
		{ 
			wIndexB = i;
			break;
		}
		
	}
	//====================================================
	fRSum = 0.0;
	for(i=wIndexR;i>=0;i--){
		fRSum += (float)hisgram_R[i];
		fRPercent = (fRSum / CountPixels) * 100;
		if(fRPercent >= 48) 
		{ 
			min_R = i;
			break;
		}
		
	}
	
	fRSum = 0.0;
	for(i=wIndexR;i<1024;i++){
		fRSum += (float)hisgram_R[i];
		fRPercent = (fRSum / CountPixels) * 100;
		if(fRPercent >= 47) 
		{ 
			max_R = i;
			break;
		}
		
	}
	//======
	fGSum = 0.0;
	for(i=wIndexG;i>=0;i--){
		fGSum += (float)hisgram_G[i];
		fGPercent = (fGSum / CountPixels) * 100;
		if(fGPercent >= 48) 
		{ 
			min_G = i;
			break;
		}
		
	}
	
	fGSum = 0.0;
	for(i=wIndexG;i<1024;i++){
		fGSum += (float)hisgram_G[i];
		fGPercent = (fGSum / CountPixels) * 100;
		if(fGPercent >= 47) 
		{ 
			max_G = i;
			break;
		}
		
	}
	
	//======
	fBSum = 0.0;
	for(i=wIndexB;i>=0;i--){
		fBSum += (float)hisgram_B[i];
		fBPercent = (fBSum / CountPixels) * 100;
		if(fBPercent >= 46)//50) 
		{ 
			min_B = i;
			break;
		}
		
	}
	
	fBSum = 0.0;
	for(i=wIndexB;i<1024;i++){
		fBSum += (float)hisgram_B[i];
		fBPercent = (fBSum / CountPixels) * 100;
		if(fBPercent >= 47)//49) 
		{ 
			max_B = i;
			break;
		}
		
	}
	
	///////////////////////////////////////////////////////////////////////////

	//Autolevel:
	sum_R = max_R- min_R;
	sum_G = max_G- min_G;
	sum_B = max_B- min_B;
	
	for(j=0; j<(int)ImageHeight; j++)
	{
		tLines = j * ImageWidth *3;
		for(i=0;i<(int)ImageWidth;i++)
		{
			R = *(pbmpdata + (tLines + i * 3 + 2));
			G = *(pbmpdata + (tLines + i * 3 + 1));
			B = *(pbmpdata + (tLines + i * 3));
			
			
			//-------------------------R
			if(sum_R == 0) R = max_R;
			else if(R<min_R){
				
				R=0;
			}
			else if((R>=min_R) && (R<=1023)){
				fmax_R = ((float)((R - min_R)*923)/(float)sum_R)+100; 
				R = (WORD)fmax_R;
				fmax_R = (fmax_R - R)*10;
				if(fmax_R >=5)
					R++;
			}
			if (R > 1023)   R = 1023;
			
			//-------------------------G	
			if(sum_G == 0) G = max_G;
			else if(G<min_G){
				
				G=0;
			}
			else if((G>=min_G) && (G<=1023)){
				fmax_G = ((float)((G - min_G)*923)/(float)sum_G)+100; 
				G = (WORD)fmax_G;
				fmax_G = (fmax_G - G)*10;
				if(fmax_G >=5)
					G++;
			}
			if (G > 1023)   G = 1023;
			
			//-------------------------B
			if(sum_B == 0) B = max_B;
			else if(B<min_R){
				
				B=0;
			}
			else if((B>=min_B) && (R<=1023)){
				fmax_B = ((float)(B - min_B)*923/(float)sum_B)+100; 

				B = (WORD)fmax_B;
				fmax_B = (fmax_B - B)*10;
				if(fmax_B >= 5)
					B++;
			}
			if (B > 1023)   B = 1023;
			
			*(pbmpdata + (tLines + i * 3 + 2)) = R;
			*(pbmpdata + (tLines + i * 3 + 1)) = G;
			*(pbmpdata + (tLines + i * 3))     = B;
			
		}//end for
	}//end for

	return;
}//end QBETDetectAutoLevel()

/**********************************************************************
Author: Jack             Date: 2005/05/14
Routine Description: 
	Change the image data and deal with auto level	
Parameters:
	lpSource: the data of image
	scanMode: the scan mode
	ScanLines: the rows of image
	BytesPerLine: the bytes of per line
Return value: 
	none
***********************************************************************/
void QBetChange(LPSTR lpSource, SCANMODE scanMode, WORD ScanLines, DWORD BytesPerLine)
{
	WORD		i,j;
	DWORD		tLines,TotalImgSize;
	WORD		R1,G1,B1,R,G,B,R2,G2,B2,QBET_RGB,PointF,PointB;
	WORD*		pwRGB;
	
	int k;
	
	DWORD ImageWidth=BytesPerLine/3;
	DWORD ImageHeight=ScanLines;
	BYTE* pbmpdata=(BYTE*)lpSource; 
	
	if (scanMode != CM_RGB24ext)
	{
		return;
	}
	
	//////////////////////////////////ALLOCATE COPY BUFFER/////////////////////////////

	TotalImgSize = ImageWidth * ImageHeight *3 *2;
	if((pwRGB = (WORD*) malloc (TotalImgSize) ) == NULL)
	{
		return ;
	}      
	

	//////////////////////////////////QBET///////////////////////////////////////////////

	for(j=0; j<ImageHeight; j++)
	{
		tLines = j * ImageWidth *3;
		for(i=0;i<ImageWidth;i++)
		{
			if(i == 0)
			{ 
				//p[i-1]
				//p[i]
				R1= R  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 2));
				G1= G  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 1));
				B1= B  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3));
				//p[i+1]
				R2 = (WORD)(BYTE)*(pbmpdata + (tLines + (i+1) * 3 + 2));
				G2 = (WORD)(BYTE)*(pbmpdata + (tLines + (i+1) * 3 + 1));
				B2 = (WORD)(BYTE)*(pbmpdata + (tLines + (i+1) * 3));
			}
			else if(i == (ImageWidth-1))
			{
				//p[i-1]
				R1 = (WORD)(BYTE)*(pbmpdata + (tLines + (i-1) * 3 + 2));
				G1 = (WORD)(BYTE)*(pbmpdata + (tLines + (i-1) * 3 + 1));
				B1 = (WORD)(BYTE)*(pbmpdata + (tLines + (i-1) * 3));
				//p[i]
				//p[i+1]
				R2= R  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 2));
				G2= G  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 1));
				B2= B  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3));
			}
			else
			{
				//p[i-1]
				R1 = (WORD)(BYTE)*(pbmpdata + (tLines + (i-1) * 3 + 2));
				G1 = (WORD)(BYTE)*(pbmpdata + (tLines + (i-1) * 3 + 1));
				B1 = (WORD)(BYTE)*(pbmpdata + (tLines + (i-1) * 3));
				//p[i]
				R  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 2));
				G  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 1));
				B  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3));
				//p[i+1]
				R2 = (WORD)(BYTE)*(pbmpdata + (tLines + (i+1) * 3 + 2));
				G2 = (WORD)(BYTE)*(pbmpdata + (tLines + (i+1) * 3 + 1));
				B2 = (WORD)(BYTE)*(pbmpdata + (tLines + (i+1) * 3));
			}
			
			R1 = R1 & 0x0003;//16i
			G1 = G1 & 0x0003;
			B1 = B1 & 0x0003;
			
			R2 = R2 & 0x0003;
			G2 = G2 & 0x0003;
			B2 = B2 & 0x0003;
			for(k=0;k<3;k++)
			{
				if(k == 0)
				{ 
					PointF = R1;
					PointB = R2;
				}
				else if(k == 1)
				{
					PointF = G1;
					PointB = G2;
				}
				else if(k == 2)
				{
					PointF = B1;
					PointB = B2;
				}
				
				switch(PointF){
				case 0:
				case 1:	
					if(PointB == 0)
						QBET_RGB = 0xFFFC;
					else if(PointB == 1)
						QBET_RGB = 0xFFFC;
					else if(PointB == 2)
						QBET_RGB = 0xFFFD;
					else if(PointB == 3)
						QBET_RGB = 0xFFFE;
					break;
				case 2:
					if(PointB == 0)
						QBET_RGB = 0xFFFD;
					else if(PointB == 1)
						QBET_RGB = 0xFFFD;
					else if(PointB == 2)
						QBET_RGB = 0xFFFF;
					else if(PointB == 3)
						QBET_RGB = 0xFFFF;
					break;
				case 3:
					if(PointB == 0)
						QBET_RGB = 0xFFFE;
					else if(PointB == 1)
						QBET_RGB = 0xFFFE;
					else if(PointB == 2)
						QBET_RGB = 0xFFFF;
					else if(PointB == 3)
						QBET_RGB = 0xFFFF;
					break;
				default:
					break;	
				}//end switch()
				
                if(k==0){
                    R = R << 2;
                    R = R + 0x0003;
                    R = R & QBET_RGB;
				}
				else if(k==1){
					G = G << 2;
                    G = G + 0x0003;
                    G = G & QBET_RGB;
				}
				else if(k==2){
                    B = B << 2;
                    B = B + 0x0003;
                    B = B & QBET_RGB;
				}//end if
				
			}//end for(k=...)
			
			*(pwRGB + (tLines + i * 3 + 2)) = R;
			*(pwRGB + (tLines + i * 3 + 1)) = G;
			*(pwRGB + (tLines + i * 3))     = B;
			
		}//end for
          
  }//end for

  ///////////////////////////////////////////////////////////////

   // Do AutoLevel:    
   QBETDetectAutoLevel(pwRGB,ImageWidth,ImageHeight); 

  //////Autolevel End/////////////QBET -0> 8bit//////////////////

   for(j=0; j<ImageHeight; j++){
	   tLines = j * ImageWidth *3;
	   for(i=0;i<ImageWidth;i++)
	   {
		   R = *(pwRGB + (tLines + i * 3 + 2)) ;
		   G = *(pwRGB + (tLines + i * 3 + 1)) ;
		   B = *(pwRGB + (tLines + i * 3))     ;
		   
		   R = R >> 2;
		   G = G >> 2;
		   B = B >> 2;
		   if(R>255) R = 255;
		   if(G>255) G = 255;
		   if(B>255) B = 255;
		   
		   *(pbmpdata + (tLines + i * 3 + 2)) = (BYTE)R;
		   *(pbmpdata + (tLines + i * 3 + 1)) = (BYTE)G;
		   *(pbmpdata + (tLines + i * 3))     = (BYTE)B;
		   
	   }//end for
	   
   }//end for
   
   //////////////////////////////////////////////////////////////////

   if(pwRGB!=NULL){
	   free(pwRGB);
   }
   
   return;
} /* end of file spicall.c*/

